// @abstract(Diablo specific unit)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(March 8, 2005)
// @lastmod(March 8, 2005)
//
unit diablout;
interface
uses rlglobal, vutil, rlnpc, rlitem, rllevel;

procedure DiabloPrepare;
procedure DiabloRandomDrop(NPC : TNPC);
procedure DiabloCellSpecial(SpecialID : Word; sx,sy : Byte );
procedure DiabloSpecialDrop(x,y : Byte);
procedure DiabloPotion(PotionEffect : Byte);
procedure DiabloRefreshShops;
procedure DiabloDone;
procedure DiabloGriswoldShop;
procedure DiabloPepinShop;


const KillCount       : DWord = 0;
      LastShop        : DWord = 0;

implementation
uses sysutils, vdebug, vmath, vini, voutput, vinput, rlmsg, rlgame, rlplayer;

const MaxItems = 200;
      MaxILevs = 32;

var   GriswoldItems   : array[1..10] of TItem;
      PepinItems      : array[1..10] of TItem;

var RandomItemTable : array[1..MaxItems] of string[30];
var RandomItemLev   : array[1..MaxILevs] of Word;

var ItemCount : DWord;

procedure ItemTableQuery(const nkey,nvalue : string);
begin
  Inc(ItemCount);
  RandomItemTable[ItemCount] := nvalue;
end;

procedure DiabloPrepare;
var Count : DWord;
begin
  for Count := 1 to 10       do GriswoldItems[Count]   := nil;
  for Count := 1 to 10       do PepinItems[Count]      := nil;
  for Count := 1 to MaxItems do RandomItemTable[Count] := '';
  for Count := 1 to MaxILevs do RandomItemLev  [Count] := 1;
  if not Database.TrySetSection('groupil1') then
    CritError('INI File -- at least one level one item must exist!');
  ItemCount := 0;
  for Count := 1 to MaxILevs do
    if Database.TrySetSection('groupil'+IntToStr(Count)) then
    begin
      Database.QuerySection(@ItemTableQuery);
      RandomItemLev  [Count] := ItemCount;
    end
    else RandomItemLev[Count] := RandomItemLev[Count-1];

  DiabloRefreshShops;
end;

procedure DiabloDropGold(x,y : byte; lev : Word);
var Gold : TItem;
begin
  Gold := TItem.Create('gold');
  Gold.Amount := RandRange(5*lev,15*lev) - 1;
  if lev > 12 then Gold.Amount := Round(Gold.Amount * 1.125);
  Game.Level.DropItem(Gold,x,y);
end;


type TSuffix = (SufWeakness,SufStrength,SufAtrophy,SufDexterity,
                SufDisease,SufVitality,SufJackal,SufFox,SufQuality,
                SufSturdiness);


procedure DiabloItemSuffix(Item : TItem; SufID : TSuffix);
begin
  case SufID of
    SufWeakness    : begin Item.Name := Item.Name + ' of weakness'; Item.StatChange[StatStr] := -RandRange(1,5); Item.Price := 0; end;
    SufStrength    : begin Item.Name := Item.Name + ' of strength'; Item.StatChange[StatStr] := RandRange(1,5); Item.Price := 10*Item.Price; end;
    SufAtrophy     : begin Item.Name := Item.Name + ' of atrophy'; Item.StatChange[StatDex] := -RandRange(1,5); Item.Price := 0; end;
    SufDexterity   : begin Item.Name := Item.Name + ' of dexterity'; Item.StatChange[StatDex] := RandRange(1,5); Item.Price := 10*Item.Price; end;
    SufDisease     : begin Item.Name := Item.Name + ' of disease'; Item.StatChange[StatVit] := -RandRange(1,5); Item.Price := 0; end;
    SufVitality    : begin Item.Name := Item.Name + ' of vitality'; Item.StatChange[StatVit] := RandRange(1,5); Item.Price := 10*Item.Price; end;
    SufJackal      : begin Item.Name := Item.Name + ' of the Jackal'; Item.StatChange[StatLife] := -RandRange(1,10); Item.Price := 0; end;
    SufFox         : begin Item.Name := Item.Name + ' of the Fox'; Item.StatChange[StatLife] := RandRange(10,15); Item.Price := 20*Item.Price; end;
    SufQuality     : begin Item.Name := Item.Name + ' of quality'; Item.StatChange[StatDmg] := RandRange(1,3); Item.Price := 25*Item.Price; end; // should be 1-2
    SufSturdiness  : begin Item.Name := Item.Name + ' of strudiness'; Item.DurMax := Round(Item.DurMax*RandRange(125,175) div 100); Item.Dur := Item.DurMax; Item.Price := 5*Item.Price; end;
  end;
end;



procedure DiabloItemSuffix(Item : TItem);
begin
  if ifArmor in Item.Flags then
    case Random(9) of
      1 : DiabloItemSuffix(Item,SufWeakness);
      2 : DiabloItemSuffix(Item,SufStrength);
      3 : DiabloItemSuffix(Item,SufAtrophy);
      4 : DiabloItemSuffix(Item,SufDexterity);
      5 : DiabloItemSuffix(Item,SufDisease);
      6 : DiabloItemSuffix(Item,SufVitality);
      7 : DiabloItemSuffix(Item,SufJackal);
      8 : DiabloItemSuffix(Item,SufFox);
      0 : DiabloItemSuffix(Item,SufSturdiness);
    end
  else
  begin
    case Random(8) of
      1 : DiabloItemSuffix(Item,SufWeakness);
      2 : DiabloItemSuffix(Item,SufStrength);
      3 : DiabloItemSuffix(Item,SufAtrophy);
      4 : DiabloItemSuffix(Item,SufDexterity);
      5 : DiabloItemSuffix(Item,SufDisease);
      6 : DiabloItemSuffix(Item,SufVitality);
      7 : DiabloItemSuffix(Item,SufQuality);
      0 : DiabloItemSuffix(Item,SufSturdiness);
    end
  end;
end;

procedure DiabloItemPrefix(Item : TItem);
begin

end;

procedure DiabloMagicItem(Item : TItem);
begin
  Include(Item.Flags,ifMagic);
  Item.Color := LightBlue;
  DiabloItemSuffix(Item);
{  case Random(100) of
    0..20  : DiabloItemPrefix(Item);
    21..83 :
    84..99 : begin DiabloItemPrefix(Item); DiabloItemSuffix(Item);end;
  end;}
end;

function DiabloRandomItem(lev : Word) : TItem;
var Item : TItem;
begin
  if Lev > MaxILevs then Lev := MaxILevs;
  Item := TItem.Create(RandomItemTable[Random(RandomItemLev[Lev])+1]);

  if (ifOneHanded in Item.Flags) or
     (ifTwoHanded in Item.Flags) or
     (ifShield in Item.Flags) or
     (ifHelm in Item.Flags) or
     (ifArmor in Item.Flags) then
     if Random(100) < (12+Item.Level) then // should be 11+0.89*(level+1)
       DiabloMagicItem(Item);
  Exit(Item);
end;

procedure DiabloDropItem(x,y : byte; lev : Word);
var Item : TItem;
begin
  Item := DiabloRandomItem(Lev);
  Game.Level.DropItem(Item,x,y);
end;

procedure DiabloCellSpecial(SpecialID : Word; sx,sy : Byte );
var Number,cnt : Byte;
begin
  case SpecialID of
    1 : case Random(10) of
          1..5 : Exit;
          6,7  : DiabloDropGold(sx,sy,Game.Level.Level);
          8    : DiabloDropItem(sx,sy,2*Game.Level.Level);
          9,0  : Game.Level.DropMonster(TNPC.Create('skeleton'),sx,sy);
        end;
    2..4 : begin
             Number := Random(SpecialID);
             if Number <> 0 then
               if Random(8) <> 0 then
               begin
                 for cnt := 1 to Number do
                   if Random(4) = 0 then
                     DiabloDropItem(sx,sy,2*Game.Level.Level)
                   else DiabloDropGold(sx,sy,Game.Level.Level);
               end
               else for cnt := 1 to Number do DiabloSpecialDrop(sx,sy);

           end;
    5 : case Random(30) of
          1..18  : Exit;
          19..21 : DiabloDropGold(sx,sy,Game.Level.Level);
          22..27 : Game.Level.DropMonster(TNPC.Create('skeleton'),sx,sy);
          28,29  : DiabloSpecialDrop(sx,sy);
          0      : DiabloDropItem(sx,sy,2*Game.Level.Level);
        end;
   201: begin
          Msg.Add('You feel yanked in a direction that never existed! Press <@<Enter@>>...');
          Msg.Draw;
          Output.Update;
          Input.GetKey([VKEY_ENTER]);
          Msg.Update;
          StairNumber := CellTownPortal;
          LevelChange := True;
          if LevelNumber = 0 then LevelNumber := Game.Player.PortalLevel
                             else LevelNumber := 0;
        end;
  end;
end;

procedure DiabloSpecialDrop(x,y : Byte);
begin
  case Random(3) of
    0 : Game.Level.DropItem(TItem.Create('softownportal'),x,y);
    1 : Game.Level.DropItem(TItem.Create('pofhealing'),x,y);
    2 : Game.Level.DropItem(TItem.Create('pofmana'),x,y);
  end;
end;


procedure DiabloRandomDrop(NPC : TNPC);
begin
  case Random(10) of
    0..5 : Exit;
    6..8 : DiabloDropGold(NPC.x,NPC.y,TLevel(NPC.Parent).Level);
    9    : DiabloDropItem(NPC.x,NPC.y,NPC.Level);
  end;
end;

procedure GriswoldCallback(Choice : Word);
begin
with Game.Player do
begin
  Output.DrawString(2,2,LightGray,'                                                                              ');
  Output.DrawString(2,2,GriswoldItems[Choice].Color,'@lItem:@> '+GriswoldItems[Choice].GetName(AName));
  UpdateStatus(GriswoldItems[Choice]);
end;
end;

procedure DiabloGriswoldShop;
var Choice : Byte;
    Size   : Byte;
    Chooser : TWgtScrollChoice;
    temp : array[1..10] of TItem;
    Window  : TWindow;
    Gold    : Byte;
begin
repeat
  Size := 0;
  for choice := 1 to 10 do if GriswoldItems[choice] <> nil then Inc(Size);
  if Size = 0 then
  begin
    Msg.Add('Griswold says "Sorry lad, got nothing to sell right now.');
    Exit;
  end;

  for Choice := 1 to 10 do temp[Choice] := nil;
  Size := 0;
  for Choice := 1 to 10 do
  begin
    if GriswoldItems[Choice] = nil then Continue;
    Inc(Size);
    temp[size] := GriswoldItems[Choice];
  end;
  for Choice := 1 to 10 do GriswoldItems[Choice] := temp[Choice];

  Output.Clear;
  Output.DrawString(2,1,DarkGray,' @<Enter@> to buy, @<Escape@> to exit.                                                ');
  Output.DrawString(2,2,LightGray,Msg.Mem1);
  Game.Level.DrawRight;
  Window := TWindow.Init(2,3,40,22,DiabloStyle);
  Window.SetTitle('Buying at Griswold ('+IntToStr(Game.Player.getGold)+' gold left)');
  Window.Show;
  Window.Draw;
  Game.Player.DrawStatus;
  Chooser := TWgtScrollChoice.Create(4,5,Black,DarkGray,White,10);
  Chooser.AddBreakChar(VKEY_ESCAPE);
  Chooser.AddCallback(@GriswoldCallback);
  for Choice := 1 to 10 do
  begin
    if GriswoldItems[choice] = nil then Continue;
    Chooser.Add(GriswoldItems[choice].GetName(BuyName));
  end;
  Choice := 0;
  if Chooser.Run <> VKEY_Escape then
  begin
    Choice := Chooser.GetChoice;
    if Game.Player.getGold < GriswoldItems[choice].Price then
      Msg.Add('Griswold says "Sorry lad, you have not enough money."')
    else
    begin
      if Game.Player.InvVolume+GriswoldItems[choice].Volume > MaxVolume then
        Msg.Add('Griswold says "Sorry lad, you have no room for that.')
      else
      begin
        Game.Player.Inv[Game.Player.InvFreeSlot] := GriswoldItems[choice];
        Gold := Game.Player.InvGoldSlot;
        Dec(Game.Player.Inv[Gold].Amount,GriswoldItems[choice].Price);
        if Game.Player.Inv[Gold].Amount = 0 then
          FreeAndNil(Game.Player.Inv[Gold]);
        Msg.Add('You buy '+GriswoldItems[choice].GetName(AName)+' for '+IntToStr(GriswoldItems[choice].Price)+' gold pieces.');
        GriswoldItems[choice] := nil;
      end;
    end;
  end;
  FreeAndNil(Chooser);
  FreeAndNil(Window);
until Choice = 0;
end;

procedure PepinCallback(Choice : Word);
begin
with Game.Player do
begin
  Output.DrawString(2,2,LightGray,'                                                                              ');
  Output.DrawString(2,2,PepinItems[Choice].Color,'@lItem:@> '+PepinItems[Choice].GetName(AName));
  UpdateStatus(PepinItems[Choice]);
end;
end;
procedure DiabloPepinShop;
var Choice : Byte;
    Size   : Byte;
    Chooser : TWgtScrollChoice;
    temp : array[1..10] of TItem;
    Window  : TWindow;
    Gold    : Byte;
begin
repeat
  Size := 0;
  for choice := 1 to 10 do if PepinItems[choice] <> nil then Inc(Size);
  if Size = 0 then
  begin
    Msg.Add('Pepin says "Sorry lad, got nothing to sell right now.');
    Exit;
  end;

  for Choice := 1 to 10 do temp[Choice] := nil;
  Size := 0;
  for Choice := 1 to 10 do
  begin
    if PepinItems[Choice] = nil then Continue;
    Inc(Size);
    temp[size] := PepinItems[Choice];
  end;
  for Choice := 1 to 10 do PepinItems[Choice] := temp[Choice];

  Output.Clear;
  Output.DrawString(2,1,DarkGray,' @<Enter@> to buy, @<Escape@> to exit.                                                ');
  Output.DrawString(2,2,LightGray,Msg.Mem1);
  Game.Level.DrawRight;
  Window := TWindow.Init(2,3,40,22,DiabloStyle);
  Window.SetTitle('Buying at Pepin ('+IntToStr(Game.Player.getGold)+' gold left)');
  Window.Show;
  Window.Draw;
  Game.Player.DrawStatus;
  Chooser := TWgtScrollChoice.Create(4,5,Black,DarkGray,White,10);
  Chooser.AddBreakChar(VKEY_ESCAPE);
  Chooser.AddCallback(@PepinCallback);
  for Choice := 1 to 10 do
  begin
    if PepinItems[choice] = nil then Continue;
    Chooser.Add(PepinItems[choice].GetName(BuyName));
  end;
  Choice := 0;
  if Chooser.Run <> VKEY_Escape then
  begin
    Choice := Chooser.GetChoice;
    if Game.Player.getGold < PepinItems[choice].Price then
      Msg.Add('Pepin says "Sorry lad, you have not enough money."')
    else
    begin
      if Game.Player.InvVolume+PepinItems[choice].Volume > MaxVolume then
        Msg.Add('Pepin says "Sorry lad, you have no room for that.')
      else
      begin
        Game.Player.Inv[Game.Player.InvFreeSlot] := PepinItems[choice];
        Gold := Game.Player.InvGoldSlot;
        Dec(Game.Player.Inv[Gold].Amount,PepinItems[choice].Price);
        if Game.Player.Inv[Gold].Amount = 0 then
          FreeAndNil(Game.Player.Inv[Gold]);
        Msg.Add('You buy '+PepinItems[choice].GetName(AName)+' for '+IntToStr(PepinItems[choice].Price)+' gold pieces.');
        PepinItems[choice] := nil;
      end;
    end;
  end;
  FreeAndNil(Chooser);
  FreeAndNil(Window);
until Choice = 0;
end;

procedure DiabloPotion(PotionEffect : Byte);
var temp : Word;
begin
with Game.Player do
case PotionEffect of
  EffectHeal       : begin temp := getLife div 4; HP := Min(HP+RandRange(temp,3*temp),getLife); Msg.Add('You feel better.'); end;
  EffectFullHeal   : begin HP := getLife; Msg.Add('You feel like new.'); end;
  EffectMana       : begin Mana := getMana; Msg.Add('You feel magical energies restoring.'); end;
  EffectFullMana   : begin Mana := getMana; Msg.Add('You feel magicaly fully restored.');end;
  EffectTownPortal : begin Msg.Add('You feel the presence of strong magic around you...'); PortalCount := 6+Random(20); end;
end;
end;

procedure DiabloRefreshShops;
var Count,Count2 : Word;
begin
  if (TurnCount = 0) or ((TurnCount - LastShop) > 10000) then
  begin
    LastShop := TurnCount;
    for Count := 1 to 10 do if GriswoldItems[Count] <> nil then FreeAndNil(GriswoldItems[Count]);
    for Count := 1 to 10 do if PepinItems[Count]  <> nil then FreeAndNil(PepinItems[Count]);
    PepinItems[1] := TItem.Create('pofhealing');
    PepinItems[2] := TItem.Create('pofhealing');
    PepinItems[3] := TItem.Create('pofhealing');
    PepinItems[4] := TItem.Create('pofhealing');
    PepinItems[5] := TItem.Create('poffullhealing');
    for Count := 1 to 6 do
      repeat
        FreeAndNil(GriswoldItems[Count]);
        GriswoldItems[Count] := DiabloRandomItem(6);
        GriswoldItems[Count].Dur := GriswoldItems[Count].DurMax;
      until (GriswoldItems[Count].Price <> 0) and (GriswoldItems[Count].Effect = 0);
  end;
end;


procedure DiabloDone;
var Count : Word;
begin
  for Count := 1 to 10 do FreeAndNil(GriswoldItems[Count]);
  for Count := 1 to 10 do FreeAndNil(PepinItems[Count]);
end;
end.

