// @abstract(Core Game Unit for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//
// This unit holds the game's main class : TGame.

unit rlgame;
interface
uses vsystem, vviewer, rllevel, rlplayer, rllua;

const MaxLevels = 16;

type

{ TGame }

TGame = class(TSystem)
       Level      : TLevel;
       LevelStore : array[0..MaxLevels] of TLevel;
       Player     : TPlayer;
       Lua        : TRLLua;
       constructor Create; override;
       procedure Load;
       procedure Intro;
       procedure Run;
       procedure ReadConfig;
       procedure ShowMortem;
       procedure Outro;
       destructor Destroy; override;
     end;

// TGame singleton.
const Game : TGame = nil;

implementation
uses sysutils, vutil, vinput, voutput, vini, vedit, vdata,
     rlglobal, rlcell, rlgen, diablout;

constructor TGame.Create;
begin
  inherited Create;
  Log('Initialized.');
  Lua := TRLLua.Create;
  if ParamStr(1) = '-god' then GodMode := True;
  ReadConfig;
end;

procedure ClassSelectCallback(Choice: word);
begin
  Output.ClearRect(60,17,75,22);
  case Choice of
    1 : Database.SetSection('warrior');
    2 : Database.SetSection('rogue');
    else exit;
  end;
  Output.DrawString(60,17,DarkGray,'@l   Level     : '+inttostr(Database.GetNumber('level')));
  Output.DrawString(60,18,DarkGray,'');
  Output.DrawString(60,19,DarkGray,'@l   Strength  : '+inttostr(Database.GetNumber('STR')));
  Output.DrawString(60,20,DarkGray,'@l   Magic     : '+inttostr(Database.GetNumber('MAG')));
  Output.DrawString(60,21,DarkGray,'@l   Dexterity : '+inttostr(Database.GetNumber('DEX')));
  Output.DrawString(60,22,DarkGray,'@l   Vitality  : '+inttostr(Database.GetNumber('VIT')));
end;

procedure TGame.Load;
var tx, ty : Word;
    Field  : VEditFieldPtr;
    ClassSelect: TWgtChoice;
    ClassSelectWnd: TWindow;

begin
  Output.HideCursor;
  Database := TDataINI.Load('data.ini');
  LoadCellSet('cellset');
  ScriptData := TIntAssocArray.Create;
  KillData := TIntAssocArray.Create;


  for tx := 0 to MaxLevels do LevelStore[tx] := nil;

  // We create a new player
  Intro;
  Output.ClearRect(1,15,40,25);
  ClassSelectWnd := TWindow.Init(41,15,80,23, DiabloStyle);
  ClassSelectWnd.SetTitle('Choose class');
  ClassSelectWnd.Show;
  ClassSelectWnd.Draw;
  ClassSelect := TWgtChoice.Create(47,18, Black, DarkGray, LightGray);
  ClassSelect.Add('Warrior');
  ClassSelect.Add(' Rogue');
  ClassSelect.AddCallback(@ClassSelectCallback);
  ClassSelect.Run;
  Case ClassSelect.GetChoice of
    1: begin Player := TPlayer.Create('warrior'); Player.Name:='Alucard'; end;
    2: begin Player := TPlayer.Create('rogue'); Player.Name:='Shana'; end;
  end;
  ClassSelect.Destroy;
  ClassSelectWnd.Destroy;
  DiabloPrepare;

  Output.Clear;
  Output.DrawString(36,12,Yellow,'Enter name');
   New(Field,Init(37,13,12,1,Brown));
  Field^.Run;
  if not Field^.LineEmpty(Field^.Return(1)) then
    Player.Name := Trim(Field^.Return(1));
  Dispose(Field,Done);
  Output.HideCursor;
  Output.Clear;
end;

procedure TGame.Intro;
begin
  Output.Clear;
  Output.DrawString(20,3,Red,   '  ####                           ####   ');
  Output.DrawString(20,4,Red,   '  #####  #    ##    ###    #    ######  ');
  Output.DrawString(20,5,Red,   '  ## ##  #   #  #   #  #   #    ##  ##  ');
  Output.DrawString(20,6,Red,   '  ## ##  #   #  #   ###    #    ##  ##  ');
  Output.DrawString(20,7,Red,   '  #@y# #@r#  #   @y#@r##@y#   #  #   #    @y#@r#  #@y#  ');
  Output.DrawString(20,8,Yellow,'  ####   #   #  #   ###    #### ######  ');
  Output.DrawString(20,9,Yellow,'  ###           #                ####   ');
  Output.DrawString(20,10,Brown,'           R O G U E L I K E            ');

  Output.DrawString(20,12,Brown,'         by Kornel Kisielewicz          ');

  Output.DrawString(20,14,Brown,'     PRE-RELEASE DEMO (7 DAY VERSION)');

  Output.DrawString(13,16,Brown,'This is the 7 day demo version,  pre-release of Diablo');
  Output.DrawString(13,17,Brown,'Roguelike.  Many  more  things are planned for it  but');
  Output.DrawString(13,18,Brown,'wether I  will  continue this project,  it all depends');
  Output.DrawString(13,19,Brown,'on you. If you like the idea, please send me an e-mail');
  Output.DrawString(13,20,Brown,'to  charon(at)magma-net.pl.  Thanks in this production');
  Output.DrawString(13,21,Brown,'goes to  Andrzej Kosnikowski  and Igor Savin  for help');
  Output.DrawString(13,22,Brown,'with aquiring Diablo-related material.                ');
  Output.DrawString(13,24,Brown,'Press <@LEnter@>> to continue...');

  Output.Update;
  Input.GetKey([VKEY_Enter]);
end;

procedure TGame.ShowMortem;
var cn : byte;
    vs : string;
    Mortem : VFileView;
begin
  VVNormalColor := LightGray;
  VVBoldColor   := White;
  VVIBarColor   := Black;
  VVITextColor  := LightGray;
  VVSub         := '@lUse arrows to scroll, PgUp, PgDown. Escape exits.';
  VVTitle := '@lPostMortem (mortem.txt)';
  if not FileExists('mortem.txt') then Exit;
  Mortem.Open('mortem.txt');
  Output.Clear;
  Mortem.Run;
  Mortem.Done;
end;

procedure TGame.Outro;
begin
  Output.Clear;
  Output.DrawString(3,3 ,LightGray,'Thank you for playing Diablo Roguelike!');
  Output.DrawString(3,4 ,LightGray,'This was just a seven day demo, keep yer eyes open for the full release!');
  Output.DrawString(3,6 ,White,'Features planned for DiabloRL:');
  Output.DrawString(3,7 ,LightGray,' -- the full range of Diablo items, monsters, uniques, prefixees and spells');
  Output.DrawString(3,8 ,LightGray,' -- catacombs, caves and hell -- each with a unique level generator');
  Output.DrawString(3,9 ,LightGray,' -- all the original Diablo quests with the original texts');
  Output.DrawString(3,10,LightGray,' -- all the hidden quest locations and uniques');
  Output.DrawString(3,11,LightGray,' -- additional quests that by Blizzard that didn''t appear in Diablo');
  Output.DrawString(3,12,LightGray,' -- full spell and missile weapon system');
  Output.DrawString(3,13,LightGray,' -- six classes -- Warrior, Mage, Rogue, Monk, Bard and Barbarian');
  Output.DrawString(3,14,LightGray,' -- maybe additional Hellfire content -- spells, items, quests, uniques.');
  Output.DrawString(3,15,LightGray,' -- Programmer''s edition -- how the author himself see''s the world of Diablo');
  Output.DrawString(3,16,LightGray,' -- proper shops, and special rooms in dungeons');
  Output.DrawString(3,19,LightGray,'Well, at least that would be if DiabloRL would be continued. It all depends');
  Output.DrawString(3,20,LightGray,'on @Lyou@>! If you wan''t this project continued then drop me a note at');
  Output.DrawString(3,21,LightGray,'@Lcharon(at)magma-net.pl@>... Comments, suggestions, death threats all welcome.');

  Output.DrawString(3,23,LightGray,'Again, thank you for the time spent playing DiabloRL.');
  Output.DrawString(3,24,LightGray,'Press @<Enter@> to quit...');

  Output.Update;
  Input.GetKey([VKEY_Enter]);
  Output.Clear;
  Output.Update;
end;

procedure TGame.Run;
var Command : Byte;
begin
  repeat
    if LevelStore[LevelNumber] = nil then
    begin
       // We create a new level
       Level := TLevel.Create('level'+IntToStr(LevelNumber));
       // Makes Level a child of TGame, so we don't have to dispose of it manualy.
       Add(Level);
       LevelStore[LevelNumber] := Level;
    end
    else
    begin
       Level := LevelStore[LevelNumber];
       GenSetLevel(Level);


       if (LevelNumber = 0) then  // if level is town level
       begin
         // Draw portal or not, depending on wether it's set up
         if Player.PortalLevel > 0 then Level.Map[64,33].Cell := CellTownPortal
                                   else Level.Map[64,33].Cell := CellFloor;
         GenFindTile([StairNumber]);
         PlayerX := GenX; PlayerY := GenY;
         if StairNumber = CellTownPortal then Inc(PlayerY);
       end
       else  // if level is dungeon level
       begin
         GenFindTile([StairNumber]);
         PlayerX := GenX; PlayerY := GenY;

         // If player came throgh portal, destroy portal
         if StairNumber = CellTownPortal then
         begin
           Level.Map[PlayerX,PlayerY].Cell := CellFloor;
           Player.PortalLevel := 0;
         end;

         // If not portal level then destroy any portals on it.
         if (LevelNumber <> Player.PortalLevel) then
         with Level do
         begin
           FindCell(CellTownPortal);
           if CellX > 0 then Map[CellX,CellY].Cell := CellFloor;
         end;
       end;
    end;

    // Makes Player a child of TLevel. We don't have to dispose of it manualy.
    // And also it sets the player in his world ;-)
    Player.Move(Level);

    // Now we can properly displace the player :D
    // GenX and GenY are taken from the generator
    Player.Displace(PlayerX,PlayerY);

    LevelChange := False;
    Player.Querry:=nil;
    repeat
      Level.TimeFlow(10);
      Inc(TurnCount);
    until GameEnd or LevelChange;
    Level.Map[Player.x,Player.y].NPC := nil;
  until GameEnd;
  if (Player.HP <= 0) or (ScriptData['BUTCHERKILLED'] = 1) then ShowMortem;
  Outro;
end;

procedure TGame.ReadConfig;
var INIFile : TINI;
begin
  INIFile := TINI.Load('diablorl.ini');
  INIFile.SetSection('KeyBindings');
  Input.Load(INIFile,@InputCommandParser);
  Input.RegisterCommand(VKEY_ESCAPE, COMMAND_ESCAPE);
  FreeAndNil(INIFile);

  if GodMode then
  begin
    Input.RegisterCommand(Ord('1'),COMMAND_GOD1);
    Input.RegisterCommand(Ord('2'),COMMAND_GOD2);
    Input.RegisterCommand(Ord('3'),COMMAND_GOD3);
    Input.RegisterCommand(Ord('4'),COMMAND_GOD4);
  end;
end;

destructor TGame.Destroy;
var Iterator : Byte;
begin
  FreeAndNil(KillData);
  FreeAndNil(ScriptData);
  FreeAndNil(Database);
  FreeAndNil(Lua);
  inherited Destroy;
  Log('Destroyed.');
end;

end.
