// @abstract(Globals for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//
// This unit holds global definiotions for the RL. No functions nor
// procedures should be here -- just constants and variables.

unit rlglobal;
interface
uses sysutils, vutil, vnode, vdata, voutput, vinput, vini, vrltools;

type TNameOutputType = (TheName,AName,PlainName,CTheName,CAName,InvName,Status1,Status2,BuyName);

type

{ TDataINI }

TDataINI = class(TINI)
       procedure  addRaw(const nKey : string; const nValue : Ansistring);override;
     end;

type TAITypeSet = set of byte;

const MapSizeX  = 100;
      MapSizeY  = 70;

      MapPosX   : byte = 1;
      MapPosY   = 2;

      MaxItems  = 60;
      MaxBeings = 60;

      MaxInvSize = 40;
      MaxEqSlots = 7;

      GoldVolume = 5000;
      MaxVolume  = 40;

      CellFloor       = 1;
      CellWall        = 2;
      CellBWall       = 3;
      CellDoorClosed  = 6;
      CellDoorOpen    = 7;
      CellDoorBlocked = 8;
      CellDoorClosed2 = 9;

      CellStairDown   = 40;
      CellStairUp     = 41;
      CellCorpse      = 70;

      CellTownPortal  = 201;

      lfLighted = 0;
      lfBlocked = 1;
      lfVisited = 2;

      mtArrow   = 1;

      GodMode     : Boolean = False;
      GameEnd     : Boolean = False;
      LevelChange : Boolean = False;
      LevelNumber : Byte = 0;
      StairNumber : Byte = 0;
      Database    : TDataINI = nil;
//      TextBase    : TTXTData = nil;
      TurnCount   : DWord = 0;
      PTurnCount  : DWord = 0;

      ScriptData  : TIntAssocArray = nil;
      KillData    : TIntAssocArray = nil;

      COMMAND_QUIT      = 1;
      COMMAND_WALKNORTH = 2;
      COMMAND_WALKSOUTH = 3;
      COMMAND_WALKEAST  = 4;
      COMMAND_WALKWEST  = 5;
      COMMAND_WALKNE    = 6;
      COMMAND_WALKSE    = 7;
      COMMAND_WALKNW    = 8;
      COMMAND_WALKSW    = 9;
      COMMAND_WAIT      = 10;
      COMMAND_PICKUP    = 11;
      COMMAND_DROP      = 12;
      COMMAND_INVENTORY = 13;
      COMMAND_EQUIPMENT = 14;
      COMMAND_ACT       = 15;
//      COMMAND_CLOSE     = 16;
      COMMAND_LOOK      = 17;
      COMMAND_FIRE      = 18;
      COMMAND_USE       = 19;
      COMMAND_HELP      = 10;
      COMMAND_PLAYERINFO= 21;
      COMMAND_SAVE      = 22;
      COMMAND_RUNMODE   = 23;
      COMMAND_STAIRSDOWN= 24;
      COMMAND_STAIRSUP  = 25;
      COMMAND_QUICKSLOT = 26;
      COMMAND_ESCAPE    = 27;

      COMMAND_GOD1 = 101;
      COMMAND_GOD2 = 102;
      COMMAND_GOD3 = 103;
      COMMAND_GOD4 = 104;

      // These need to be high to not to be in conflict with
      // inventory KeyCodes. They also need to be sequential.
      COMMAND_QUICKSLOT1= 231;
      COMMAND_QUICKSLOT2= 232;
      COMMAND_QUICKSLOT3= 233;
      COMMAND_QUICKSLOT4= 234;
      COMMAND_QUICKSLOT5= 235;
      COMMAND_QUICKSLOT6= 236;
      COMMAND_QUICKSLOT7= 237;
      COMMAND_QUICKSLOT8= 238;


      COMMAND_QUICKSLOTSCOUNT = 8;
      COMMAND_QUICKSLOTS= [COMMAND_QUICKSLOT1..COMMAND_QUICKSLOT8];
      COMMANDS_MOVE     = [COMMAND_WALKNORTH..COMMAND_WALKSW];

      SlotHead    = 1;
      SlotAmulet  = 2;
      SlotTorso   = 3;
      SlotRHand   = 4;
      SlotLHand   = 5;
      SlotRRing   = 6;
      SlotLRing   = 7;

// Defines a window style.
type TWindowStyle = record
       Framed     : Boolean;
       BackColor  : Byte;
       FrameColor : Byte;
       TitleColor : Byte;
       FrameChars : string[8]; {-L---}
     end;


// Main window class. Can be used as is, or inherited from.
type TWindow = class(TVObject)
       // Set basic window parameters.
       constructor Init(nx1,ny1,nx2,ny2 : byte; nStyle : TWindowStyle); virtual;
       // Sets and activates a window title. Passing '' deactivates title.
       procedure SetTitle(nTitle : string);
       // Sets the window as visible.
       procedure Show;
       // Sets the window as not visible.
       procedure Hide;
       // Draws the window and it's contents.
       procedure Draw;
       // sets wether the window should clear it's background.
       procedure setClear(newClear : boolean);
       // Disposes the window and it's components.
       destructor Destroy; override;
       private
       procedure Frame;
       x1,y1,x2,y2 : Byte;
       Visible     : Boolean;
       Active      : Boolean;
       Clearing    : Boolean;
       Title       : String[70];
       Style       : TWindowStyle;
     end;


type TChoiceCallback = procedure(Choice : Word);
// Implements a widget for chioce list.
type TWgtChoice = class(TVObject)
       // Initializes the choice list.
       constructor Create(nx1,ny1 : Byte; nFieldColor : Byte; nTextColor : Byte; nBoldColor : Byte); virtual;
       // Runs the input waiting for a choice to be pressed.
       // Returns 0 or a breakKey if pressed.
       function Run : Word; virtual;
       // Returns the choice.
       function GetChoice : Word;
       // Draw the widget.
       procedure Draw; virtual;
       // Adds a choice to the choice list. The order in which
       // the choices are added IS revelant.
       procedure Add(const ChoiceString : string); virtual;
       // Adds callback procedure
       procedure AddCallback(newCallback : TChoiceCallback); virtual;
       // Frees all allocated memory.
       destructor Destroy; override;
       // Adds a break char.
       procedure AddBreakChar(Key : byte);
       protected
       CallBack  : TChoiceCallback;
       TextColor : byte;
       BoldColor : byte;
       FieldColor: byte;
       Choice    : Word;
       Choices   : Word;
       MaxLen    : byte;
       Overriden : Boolean;
       Data      : TStrArray;
       BreakChars: TKeyFilter;
       x1, y1, y2: Byte;
     end;

// Implements a widget for chioce list -- scrolling
type TWgtScrollChoice = class(TWgtChoice)
       // Initializes the choice list.
       constructor Create(nx1,ny1 : Byte; nFieldColor : Byte; nTextColor : Byte; nBoldColor : Byte; MaxHeight : byte); virtual;
       // Draw the widget.
       procedure Add(const ChoiceString : string); override;
       procedure Draw; override;
       protected
       mHeight : Byte;
     end;


const DiabloStyle : TWindowStyle = (
        Framed     : True;
        BackColor  : Black;
        FrameColor : DarkGray;
        TitleColor : LightGray;
        FrameChars : 'ڿĳ';
      );

{$INCLUDE ../bin/flags.ini}

{$INCLUDE ../bin/defines.ini}

function InputCommandParser(const Token : string) : byte;
function RandRange(RangeMin,RangeMax : LongInt) : LongInt;
function CapLet(name : string) : string;
function DPad(str : string; len : Byte) : string;
function KlassName(Klass : Byte) : string;
function KlassPronoun(Klass : Byte) : string;
function CommandDirection(Command: byte): byte;

const ExpTable : array[1..50] of DWord = (
      0,          2000,       4620,       8040,       12489,
      18258,      25712,      35309,      47622,      63364,
      83419,      108876,     141086,     181683,     231075,
      313656,     424068,     571190,     766597,     1025154,
      1366227,    1814568,    2401895,    3168651,    4166200,
      5459523,    7130496,    9281874,    12042092,   15571031,
      20066900,   25744405,   32994399,   42095202,   53525811,
      67831218,   85670061,   107834823,  135274799,  196122009,
      210720231,  261657253,  323800420,  399335440,  490808349,
      601170414,  733825617,  892680222,  1082908612, 1310707109
      );


implementation


function CommandDirection(Command: byte): byte;
begin
  case Command of
      COMMAND_WALKNORTH : exit(DIR_UP);
      COMMAND_WALKSOUTH : exit(DIR_DOWN);
      COMMAND_WALKEAST  : exit(DIR_RIGHT);
      COMMAND_WALKWEST  : exit(DIR_LEFT);
      COMMAND_WALKNE    : exit(DIR_UPRIGHT);
      COMMAND_WALKSE    : exit(DIR_DOWNRIGHT);
      COMMAND_WALKNW    : exit(DIR_UPLEFT);
      COMMAND_WALKSW    : exit(DIR_DOWNLEFT);
  else
     exit(DIR_NONE);
  end;
end;

function KlassName(Klass : Byte) : string;
begin
  case Klass of
    KlassWarrior  : Exit('Warrior');
    KlassRogue    : Exit('Rogue');
    KlassSorceror : Exit('Sorceror');
  end;
end;

function KlassPronoun(Klass : Byte) : string;
begin
  if Klass in [KlassSorceror,KlassWarrior] then Exit('He') else Exit('She');
end;

function DPad(str : string; len : Byte) : string;
begin
  if Output.Length(str) = len then Exit(str);
  if Output.Length(str) > len then
  begin
    while Output.Length(str) > Len do
      Delete(str,Length(str),1);
    Exit(str);
  end;
  while Output.length(str) < len do str := str + ' ';
  Exit(str);
end;

function CapLet(name : string) : string;
begin
  name[1] := UpCase(name[1]);
end;

function InputCommandParser(const Token : string) : byte;
begin
  if Token = 'WALKNORTH'   then Exit(COMMAND_WALKNORTH);
  if Token = 'WALKSOUTH'   then Exit(COMMAND_WALKSOUTH);
  if Token = 'WALKEAST'    then Exit(COMMAND_WALKEAST);
  if Token = 'WALKWEST'    then Exit(COMMAND_WALKWEST);
  if Token = 'WALKNE'      then Exit(COMMAND_WALKNE);
  if Token = 'WALKSE'      then Exit(COMMAND_WALKSE);
  if Token = 'WALKNW'      then Exit(COMMAND_WALKNW);
  if Token = 'WALKSW'      then Exit(COMMAND_WALKSW);
  if Token = 'WAIT'        then Exit(COMMAND_WAIT);
  if Token = 'PICKUP'      then Exit(COMMAND_PICKUP);
  if Token = 'DROP'        then Exit(COMMAND_DROP);
  if Token = 'INVENTORY'   then Exit(COMMAND_INVENTORY);
  if Token = 'EQUIPMENT'   then Exit(COMMAND_EQUIPMENT);
  if Token = 'ACT'         then Exit(COMMAND_ACT);
  if Token = 'LOOK'        then Exit(COMMAND_LOOK);
  if Token = 'FIRE'        then Exit(COMMAND_FIRE);
  if Token = 'USE'         then Exit(COMMAND_USE);
  if Token = 'HELP'        then Exit(COMMAND_HELP);
  if Token = 'PLAYERINFO'  then Exit(COMMAND_PLAYERINFO);
  if Token = 'SAVE'        then Exit(COMMAND_SAVE);
  if Token = 'QUIT'        then Exit(COMMAND_QUIT);
  if Token = 'RUNMODE'     then Exit(COMMAND_RUNMODE);
  if Token = 'STAIRSDOWN'  then Exit(COMMAND_STAIRSDOWN);
  if Token = 'STAIRSUP'    then Exit(COMMAND_STAIRSUP);
  if Token = 'QUICKSLOT'   then Exit(COMMAND_QUICKSLOT);
  if Token = 'QUICKSLOT1'  then Exit(COMMAND_QUICKSLOT1);
  if Token = 'QUICKSLOT2'  then Exit(COMMAND_QUICKSLOT2);
  if Token = 'QUICKSLOT3'  then Exit(COMMAND_QUICKSLOT3);
  if Token = 'QUICKSLOT4'  then Exit(COMMAND_QUICKSLOT4);
  if Token = 'QUICKSLOT5'  then Exit(COMMAND_QUICKSLOT5);
  if Token = 'QUICKSLOT6'  then Exit(COMMAND_QUICKSLOT6);
  if Token = 'QUICKSLOT7'  then Exit(COMMAND_QUICKSLOT7);
  if Token = 'QUICKSLOT8'  then Exit(COMMAND_QUICKSLOT8);
  Exit(0);
end;

function RandRange(RangeMin,RangeMax : LongInt) : LongInt;
begin
  if RangeMin > RangeMax then Exit(RangeMin)
                         else Exit(Random(RangeMax-RangeMin+1)+RangeMin);
end;

constructor TWindow.Init(nx1,ny1,nx2,ny2 : byte; nStyle : TWindowStyle);
begin
  x1 := nx1; x2 := nx2;
  y1 := ny1; y2 := ny2;
  Style := nStyle;
  Visible := False;
  Active := False;
  Title := '';
  Clearing := True;
end;

procedure TWindow.setClear(newClear : boolean);
begin
  Clearing := newClear;
end;

procedure TWindow.SetTitle(nTitle : string);
begin
  Title := nTitle;
  if Title <> '' then Title := ' '+Title+' ';
end;

procedure TWindow.Show;
begin
  Visible := True;
end;

procedure TWindow.Hide;
begin
  Visible := False;
end;

procedure TWindow.Draw;
begin
  if not Visible then Exit;
  if Clearing then Output.ClearRectColor(x1,y1,x2,y2,Style.BackColor);
  if Style.Framed then Frame;
  if Title <> '' then
    Output.DrawString(x1+((x2-x1) div 2)-(Output.Length(Title) div 2),y1,Style.TitleColor,Title);
end;

procedure TWindow.Frame;
var x : byte;
begin
  Output.DrawChar(x1,y1,Style.FrameColor,Style.FrameChars[1]);
  Output.DrawChar(x2,y1,Style.FrameColor,Style.FrameChars[2]);
  Output.DrawChar(x1,y2,Style.FrameColor,Style.FrameChars[3]);
  Output.DrawChar(x2,y2,Style.FrameColor,Style.FrameChars[4]);
  for x := x1+1 to x2-1 do
  begin
    Output.DrawChar(x,y1,Style.FrameColor,Style.FrameChars[5]);
    Output.DrawChar(x,y2,Style.FrameColor,Style.FrameChars[6]);
  end;
  for x := y1+1 to y2-1 do
  begin
    Output.DrawChar(x1,x,Style.FrameColor,Style.FrameChars[7]);
    Output.DrawChar(x2,x,Style.FrameColor,Style.FrameChars[8]);
  end;
end;

destructor TWindow.Destroy;
begin
  LOG('Disposing.');
  inherited Destroy;
  LOG('Done.');
end;

constructor TWgtChoice.Create(nx1,ny1 : Byte; nFieldColor : Byte; nTextColor : Byte; nBoldColor : Byte);
begin
  FieldColor := nFieldColor;
  x1 := nx1;
  y1 := ny1;

  Data := TStrArray.Create(250);
  TextColor := nTextColor;
  BoldColor := nBoldColor;
  Choice    := 1;
  Choices   := 0;
  MaxLen    := 0;
  BreakChars:= [];
  Overriden := False;
  Callback := nil;
end;

function TWgtChoice.Run : Word;
var Key : Byte;
begin
  Draw;
  Output.Update;
  repeat
    if Assigned(Callback) then Callback(Choice);
    Key := Input.GetKey([VKEY_UP,VKEY_DOWN,VKEY_ENTER]+BreakChars);
    case Key of
      VKEY_UP   : if Choice > 1       then Dec(Choice);
      VKEY_DOWN : if Choice < Choices then Inc(Choice);
    end;
    if Key in BreakChars then Break;
    Draw;
    if Assigned(Callback) then Callback(Choice);
    Output.Update;
  until (key = VKEY_ENTER);
  if Key in BreakChars then Run := Key else Run := 0;
end;

procedure TWgtChoice.Draw;
var count : byte;
begin
  Output.ClearRectColor(x1,y1,x1+MaxLen,y2,FieldColor);
  if Overriden then Exit;
  for count := 1 to Choices do
  if count = Choice then Output.DrawString(x1,y1+count-1,BoldColor+16*FieldColor,Data[count])
                    else Output.DrawString(x1,y1+count-1,TextColor+16*FieldColor,Data[count]);
end;

function TWgtChoice.GetChoice : Word;
begin
  Exit(Choice);
end;

procedure TWgtChoice.Add(const ChoiceString : string);
begin
  Inc(Choices);
  if choices > 24 then CritError('Choices overflow > 25!');
  if Output.Length(ChoiceString) > MaxLen then MaxLen := Output.Length(ChoiceString);
  Data[Choices] := ChoiceString;
  y2 := y1+Choices-1;
end;

procedure TWgtChoice.AddBreakChar(Key : byte);
begin
  Include(BreakChars,Key);
end;

procedure TWgtChoice.AddCallback(newCallback : TChoiceCallback);
begin
  Callback := newCallback;
end;


destructor TWgtChoice.Destroy;
begin
  FreeAndNil(Data);
  inherited Destroy;
end;

// Initializes the choice list.
constructor TWgtScrollChoice.Create(nx1,ny1 : Byte; nFieldColor : Byte; nTextColor : Byte; nBoldColor : Byte; MaxHeight : byte);
begin
  inherited Create(nx1,ny1,nFieldColor,nTextColor,nBoldColor);
  mHeight := MaxHeight;
  Overriden := True;
end;

procedure TWgtScrollChoice.Draw;
var count  : byte;
    scroll : Integer;
begin
  inherited Draw;
  scroll := choice - (mHeight div 2);
  if scroll < 0 then scroll := 0;
  if scroll+mHeight > choices then Scroll := Choices-mHeight;
  if mHeight >= Choices then scroll := 0;
  for count := scroll+1 to scroll+mHeight do
  if count = Choice then Output.DrawString(x1,y1+count-1-scroll,BoldColor+16*FieldColor,Data[count])
                    else Output.DrawString(x1,y1+count-1-scroll,TextColor+16*FieldColor,Data[count]);
end;

procedure TWgtScrollChoice.Add(const ChoiceString : string);
begin
  Inc(Choices);
  if Output.Length(ChoiceString) > MaxLen then MaxLen := Output.Length(ChoiceString);
  Data[Choices] := ChoiceString;
  y2 := y1+mHeight-1;
end;

procedure TDataINI.addRaw(const nKey : string; const nValue : Ansistring);
var Count   : DWord;
    Group   : String;
    SName   : String;
    Section : TEntryAssocArray;
begin
  if UpCase(nKey) = 'GROUP' then
  begin
    Section := Selected;
    Count := 0;
    SName := FLastEntryName;
    repeat
      Inc(Count);
      Group :=  Trim(Parameter(nValue,Count,','));
      if Group = '' then Break;
      if not trySetSection(Group) then AddSection(Group);
      Add(Group+IntToStr(Selected.getCount),SName,Selected);
    until Group = '';
    Selected := Section;
    exit;
  end;
  inherited addRaw(nKey, nValue);
end;

end.
