// @abstract(Game Object for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//
// This unit holds the game's generic game object class : TGameObject.
// The purpose is for all the object to share a common Save/Load interface.

unit rlgobj;
interface
uses vnode, vutil, rlglobal;

type TGameObject = class(TNode)
       Flags : TFlags;
       Name  : string[30];
       tid   : string[30];
       // General constructor
       constructor Create(const thingID :string); virtual;
       // GetName returns the name of the thing -- to be overridden
       // GetName output MAY include colors -- for plain name use Name
       // directly.
       function GetName(outputType : TNameOutputType) : string; virtual;
       // This general procedure handles time-flow. Time is the amount
       // of time passed since last call. In GameObject it just propagates
       // TimeFlow to children and siblings.
       // Note: Due to this fact TGameObject siblings and children MUST
       // be TGameObjects.
       procedure TimeFlow(time : LongInt); virtual;
     end;

implementation
uses SysUtils;

constructor TGameObject.Create(const thingID :string);
var temp : Byte;
begin
  inherited Create;
  tid := LowerCase(thingID);
  if not Database.TrySetSection(thingID) then CritError('Unknown ThingID -- "'+thingID+'"');
  Name := Database.GetString('name');
  if Database.Defined('flags') then Flags := Database.GetFlags('flags');
end;

function TGameObject.GetName(outputType : TNameOutputType) : string;
begin
  case outputType of
    TheName    : Exit('the '+Name);
    AName      : Exit('a '+Name);
    PlainName  : Exit(Name);
    CTheName   : Exit('The '+Name);
    CAName     : Exit('A '+Name);
    InvName    : Exit(name);
  end;
end;




procedure TGameObject.TimeFlow(time : LongInt);
var Scan : TNode;
begin
  if Child <> nil then
  begin
    Scan := Child;
    repeat
      TGameObject(Scan).TimeFlow(time);
      Scan := Scan.Next;
    until Scan = Child;
  end;
end;

end.
