// @abstract(Item for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//

unit rlitem;
interface
uses rlthing, rlglobal;

type TItem = class(TThing)
       Volume     : Byte;
       Amount     : DWord;
       StatReq    : array[1..3] of Byte;
       StatChange : array[1..8] of ShortInt;
       DurMax     : Word;
       Dur        : Word;
       DmgMin     : Byte;
       DmgMax     : Byte;
       Effect     : Word;
       Price      : Word;
       // Overriden constructor.
       constructor Create(const thingID :string); override;
       // TimeFlow. See TGameObject. Not Needed for Items in DiabloRL.
       procedure TimeFlow(time : LongInt); override;
       // Returns item name
       function GetName(outputType : TNameOutputType) : string; override;
     end;

implementation

uses sysutils, vutil,rlplayer;

// Overriden constructor.
constructor TItem.Create(const thingID :string);
var count : byte;
begin
  // Most importantly set the section of the database by inheritance of
  // TThing.
  inherited Create(thingID);
  for Count := 1 to 3 do StatReq[Count] := 0;
  for Count := 1 to 6 do StatChange[Count] := 0;
  DurMax := 0; Dur := 0;
  DmgMin := 0; DmgMax := 0;
  Effect := 0;
  Price := 0;
  if Database.Defined('volume') then Volume := Database.GetNumber('volume') else Volume := 1;
  if Database.Defined('minstr') then StatReq[StatStr] := Database.GetNumber('minstr');
  if Database.Defined('minmag') then StatReq[StatMag] := Database.GetNumber('minmag');
  if Database.Defined('mindex') then StatReq[StatDex] := Database.GetNumber('mindex');
  if Database.Defined('effect') then Effect := Database.GetNumber('effect');
  if Database.Defined('price') then Price := Database.GetNumber('price');
  
  if Database.Defined('damagemin') then
  begin
    DmgMin := Database.GetNumber('damagemin');
    DmgMax := Database.GetNumber('damagemax');
  end;

  if Database.Defined('armormin') then
       StatChange[StatAC]  := RandRange(Database.GetNumber('armormin'),Database.GetNumber('armormax'));
  if Database.Defined('durability') then
  begin
    DurMax := Database.GetNumber('durability');
    Dur    := RandRange(1+(DurMax div 4),(DurMax div 4)*3);
  end;
  Amount := 1;
end;

function TItem.GetName(outputType : TNameOutputType) : string;
var temp : string[70];
  function VS(l : LongInt) : string;
  begin if l > 0 then Exit('+'+IntToStr(l)) else Exit(IntToStr(l)); end;
begin
  if OutputType = Status2 then
  begin
    if StatChange[StatStr]  <> 0 then Exit('Strength '+VS(StatChange[StatStr]));
    if StatChange[StatMag]  <> 0 then Exit('Magic '+VS(StatChange[StatMag]));
    if StatChange[StatVit]  <> 0 then Exit('Vitality '+VS(StatChange[StatVit]));
    if StatChange[StatDex]  <> 0 then Exit('Dexterity '+VS(StatChange[StatDex]));
    if StatChange[StatLife] <> 0 then Exit('Life '+VS(StatChange[StatLife]));
    if StatChange[StatMana] <> 0 then Exit('Mana '+VS(StatChange[StatMana]));
    if StatChange[StatDmg]  <> 0 then Exit('Damage '+VS(StatChange[StatDmg]));
    Exit('');
  end;
  if Amount > 1 then
  case outputType of
    TheName    : Exit(IntToStr(Amount)+' '+Name);
    AName      : Exit('a pile of '+IntToStr(Amount)+' '+Name);
    PlainName  : Exit(IntToStr(Amount)+' '+Name);
    CTheName   : Exit(IntToStr(Amount)+' '+Name);
    CAName     : Exit('A pile of '+IntToStr(Amount)+' '+Name);
    InvName    : Exit(DPad(IntToStr(Amount)+' '+name,33)+' '+IntToStr(Volume));
    Status1    : Exit(IntToStr(Amount)+' '+Name);
  end;
  temp := name;
  
  if StatChange[StatAC] <> 0 then
    if OutputType = PlainName then temp := temp+' ['+IntToStr(StatChange[StatAC])+']'
                              else temp := temp+' [@<'+IntToStr(StatChange[StatAC])+'@>]';
  if DmgMin <> 0 then
    if OutputType = PlainName then temp := temp+' ('+IntToStr(DmgMin)+'-'+IntToStr(DmgMax)+')'
                              else temp := temp+' (@<'+IntToStr(DmgMin)+'-'+IntToStr(DmgMax)+'@>)';
  if OutputType = Status1 then if DurMax <> 0 then temp := temp+' (Dur @<'+IntToStr(Dur)+'@>/@<'+IntToStr(DurMax)+'@>)';

  case outputType of
    TheName    : temp := 'the '+temp;
    AName      : temp := 'a '+temp;
    CTheName   : temp := 'The '+temp;
    CAName     : temp := 'A '+temp;
    InvName    : temp := DPad(temp,33)+' '+IntToStr(Volume);
    BuyName    : temp := DPad(temp,29)+' '+IntToStr(Price);
  end;
  Exit(temp);
end;

procedure TItem.TimeFlow(time : LongInt);
begin
  inherited TimeFlow(time);
end;

end.
