// @abstract(NPC for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//

unit rlnpc;
interface
uses rlthing, vrltools, rlglobal;

type

{ TNPC }

TNPC = class(TThing)
       SpeedCount : Integer;
       AI         : Byte;
       Corpse     : Byte;
       tx,ty      : Word;
       AC         : Word;
       ToHit      : Word;
       HP         : Integer;
       HPMax      : Word;
       DmgMin     : Word;
       DmgMax     : Word;
       ExpValue   : Word;
       SpdAtk     : Byte;
       SpdMov     : Byte;
       SpdHit     : Byte;
       Recovery   : Byte;
       // Overriden constructor -- sets x,y to (1,1).
       constructor Create(const thingID :string); override;
       procedure Action; virtual;
       // Opening doors/Closing doors/Opening Sacrophagus/etc...
       procedure ActivateCell(cx,cy : word);
       // Attack Player
       procedure Attack(NPC : TNPC); virtual;
       // TimeFlow. See GameObject.
       procedure TimeFlow(time : LongInt); override;
       // returns DmgMin
       function getDmgMin : Word; virtual;
       // returns DmgMax
       function getDmgMax : Word; virtual;
       // returns AC
       function getAC : Word; virtual;
       // returns ToHit
       function getToHit : Word; virtual;
       // returns HPMax
       function getLife : Word; virtual;
       procedure DrawHP;
       // Damage application.
       procedure applyDamage(Damage : DWord);
       // Death procedure -- does all cleanup.
       procedure Die; virtual;
       // Displays the message only if the given NPC is a Player.
       procedure PlayerMsg(const str : string);
       // Checks wether this given NPC is the player
       function IsPlayer : boolean;
       // Returns true if the monster is in the vision field of the player.
       function Visible : boolean;
       // Returns amount of experience gained by killer of killerLevel
       function getExpValue(killerLevel : Byte) : DWord;
       // Seeks given position
       procedure Seek(sx,sy : Word);
       // Sends a missile
       procedure SendMissile(tgx, tgy : Byte; mtype : Byte);
       //Seek for target
       function ClosestTarget(AISet: TAITypeSet; Radius : Byte):TNPC;
     end;

implementation
uses vmath, vutil, voutput, vinput, vflags,
     rlgame,rlcell,rllevel,rlmsg, rlplayer,
     diablout, sysutils;

procedure TNPC.DrawHP;
var s:string;
begin
  s:=name;
  Output.ClearRect(10,24,71,25);
  if isPlayer then exit;
  Output.Drawstring(40-(length(s) div 2),24,LightGray, s);
  if AI=AIMonster then
  begin
    Output.DrawString(34,23,DarkGray, '[@r'+PadSpace('',((hp*10) div hpmax),'#')+'@d'+PadSpace('',10-((hp*10) div hpmax),'.')+']');
    Output.Drawstring(33,25,LightGray, 'Total kills : '+inttostr(KillData[tid]));
  end;
end;

function TNPC.ClosestTarget(AISet: TAITypeSet; Radius : Byte):TNPC;
var scx, scy, t, ran : Word;
begin
  ran:=1000;
  for scx := Max(1,x-Radius-1) to Min(MapSizeX,x+Radius+1) do
    for scy := Max(1,y-Radius-1) to Min(MapSizeY,y+Radius+1) do
      if not (TLevel(Parent).CellCheck(scx,scy, [noMonsters])) then
        if (TLevel(Parent).Map[scx,scy].NPC.visible) then
          if (TLevel(Parent).Map[scx,scy].NPC.AI in AISet) then
          begin
            t := Distance(scx,scy,x,y);
            if t <= ran then
            begin
              ran:=t;
              tx := scx;
              ty := scy;
            end;
          end;
  if ran = 1000 then
    exit(nil)
  else
    exit(TLevel(Parent).Map[tx,ty].NPC);
end;

constructor TNPC.Create(const thingID :string);
begin
  // Set other atributes and find given thingID
  inherited Create(thingID);
  if Database.Defined('corpse') then Corpse := Database.GetNumber('corpse')
                                else Corpse := 0;
  AI := Database.GetNumber('AI');
  HPMax := RandRange(Database.GetNumber('hpmin'),Database.GetNumber('hpmax'));
  AC := Database.GetNumber('ac');
  DmgMin := Database.GetNumber('dmgmin');
  DmgMax := Database.GetNumber('dmgmax');
  ExpValue := Database.GetNumber('expvalue');
  SpdAtk := Database.GetNumber('spdatk');
  SpdMov := Database.GetNumber('spdmov');
  SpdHit := Database.GetNumber('spdhit');
  ToHit := Database.GetNumber('tohit');
  if Game.Lua.FunctionExists(LowerCase(thingID)+'_OnKill') then Include(Flags,nfOnKill);
  if Game.Lua.FunctionExists(LowerCase(thingID)+'_OnTalk') then Include(Flags,nfOnTalk);
  HP := HPMax;
  SpeedCount := 90-Random(40);
  Recovery := 0;
end;

// returns DmgMin
function TNPC.getDmgMin : Word;
begin
  Exit(DmgMin);
end;

// returns DmgMax
function TNPC.getDmgMax : Word;
begin
  Exit(DmgMax);
end;

function TNPC.getAC : Word;
begin
  Exit(AC);
end;

function TNPC.getToHit : Word;
begin
  Exit(ToHit);
end;

function TNPC.getLife : Word;
begin
  Exit(HPMax);
end;


function TNPC.IsPlayer : boolean;
begin
  Exit(AI = AIPlayer);
end;

function TNPC.Visible : boolean;
begin
  Exit(FlagSet(TLevel(Parent).Map[x,y].Light,lfLighted));
end;

// Attack Player
procedure TNPC.Attack(NPC : TNPC);
var HitChance  : Integer;
    Damage     : Integer;
begin
  if NPC = nil then Exit;
    Dec(SpeedCount,SpdAtk);

  HitChance := 30+getToHit+2*Level;
  HitChance := HitChance-(NPC.getAC+2*NPC.Level);

  if HitChance < 15  then HitChance := 15;
  case TLevel(Parent).Level of
    14 : if HitChance < 20  then HitChance := 20;
    15 : if HitChance < 25  then HitChance := 25;
    16 : if HitChance < 30  then HitChance := 30;
  end;

  if Random(100) >= HitChance then
  begin
    Msg.Add(GetName(CTheName)+' misses you.');
    Exit;
  end;

  if NPC.IsPlayer then
    with TPlayer(NPC) do
      if Recovery = 0 then
      if Eq[SlotLHand] <> nil then
        if Random(100) < (getBlock+2*(Level-Self.Level)) then
        begin
          Msg.Add('You block the blow of '+Self.GetName(TheName)+'.');
          Recovery := 10;
          durabilityCheck(SlotLHand);
          Exit;
        end;

  Damage := RandRange(DmgMin,DmgMax);

  Msg.Add(GetName(CTheName)+' hits you.');

  // Hack, should be only one affected.
  with TPlayer(NPC) do
  begin
    if Eq[SlotTorso] <> nil then if Random(4)=0 then durabilityCheck(SlotTorso);
    if Eq[SlotHead]  <> nil then if Random(4)=0 then durabilityCheck(SlotHead);
  end;

// Warning -- NPC may be invalid afterwards.
  NPC.ApplyDamage(Damage);
end;


procedure TNPC.ActivateCell(cx,cy : word);
var CellID : Byte;
begin
  with TLevel(Parent) do
  begin
    if (Map[cx,cy].NPC <> nil) or (Map[cx,cy].Item <> nil) then
    begin
      PlayerMsg('There''s something blocking the way.');
      Exit;
    end;
    cellID := Map[cx,cy].Cell;
    if Cells[cellID].ActTo <> 0 then
    begin
      case cellID of
        CellDoorOpen    : PlayerMsg('You close the door.');
        CellDoorBlocked : PlayerMsg('The door is blocked.');
        CellDoorClosed  : PlayerMsg('You open the door.');
        CellDoorClosed2 : PlayerMsg('You open the door. Click.');
        20              : PlayerMsg('You open the sacrophagus.');
        21              : PlayerMsg('You open the small chest.');
        22              : PlayerMsg('You open the medium chest.');
        23              : PlayerMsg('You open the large chest.');
        28              : PlayerMsg('You destroy the barrel.');
      end;
      Map[cx,cy].Cell := Cells[cellID].ActTo;
      if Cells[cellID].Special <> 0 then DiabloCellSpecial(Cells[CellID].Special,cx,cy);
      Exit;
    end;
  end;
  PlayerMsg('There''s nothing to act upon there.');
end;


procedure TNPC.applyDamage(Damage : DWord);
begin
  Dec(HP,Damage);
  if HP > Level+3 then Recovery := SpdHit;
  // warning, no access afterwards
  if HP <= 0 then Die;
end;


function TNPC.getExpValue(killerLevel : Byte) : DWord;
var value : LongInt;
begin
  Value := Round(ExpValue*(1.0+0.1*(Level-killerLevel)));
  if Value < 0 then Value := 0;
  Exit(Value);
end;

procedure TNPC.TimeFlow(time : LongInt);
begin
  if GameEnd then Exit;
  // Do your action
  Inc(SpeedCount,Max(0,time-Recovery));
  Recovery := Max(0,Recovery-Time);
  while (SpeedCount >= 100) and (GameEnd = False) do Action;
  // Pass time to siblings and children.
  inherited TimeFlow(time);
end;

procedure TNPC.Die;
begin
  Inc(KillCount);
  KillData[tid]:=KillData[tid]+1;
  if self = game.player.querry then game.player.querry:=nil;
  if Visible then Msg.Add(GetName(CTheName)+' dies.');
  DiabloRandomDrop(Self);
  Msg.Add('You gain '+IntToStr(getExpValue(Game.Player.Level))+' experience.');
  Game.Player.AddExp(getExpValue(Game.Player.Level));
  TLevel(Parent).Map[x,y].NPC := nil;
  if Corpse <> 0 then
    if Cells[TLevel(Parent).Map[x,y].cell].canch then
      TLevel(Parent).Map[x,y].cell  := Corpse;
  if nfOnKill in Flags then Game.Lua.Execute(tid+'_OnKill');
  Detach;
  Destroy;
end;

procedure TNPC.PlayerMsg(const str : string);
begin
  if IsPlayer then Msg.Add(str);
end;

procedure TNPC.Seek(sx,sy : Word);
var rx,ry       : Word;
    mx,my       : ShortInt;
    MoveResult  : byte;
  function TryMove(mvx,mvy: word) : Byte;
  begin with TLevel(Parent) do begin
    if Map[mvx,mvy].NPC <> nil then Exit(100);
    if not Cells[Map[mvx,mvy].Cell].Pass then Exit(100);
    if Cells[Map[mvx,mvy].Cell].Special <> 0 then Exit(100);
    rx := mvx; ry := mvy;
    Exit(0);
  end; end;
begin
  mx := Sgn(sx-x);
  my := Sgn(sy-y);

  MoveResult := TryMove(x+mx,y+my);

  if MoveResult = 100 then
    if mx = 0 then
      MoveResult := TryMove(x+(Random(2)*2-1),y+my)
    else
      MoveResult := TryMove(x+mx,y);
  if MoveResult = 100 then
    if my = 0 then
      MoveResult := TryMove(x+mx,y+(Random(2)*2-1))
    else
      MoveResult := TryMove(x,y+my);

  if MoveResult = 0 then
  begin
    Dec(SpeedCount,SpdMov);
    Displace(rx,ry);
    Exit;
  end;
end;

procedure TNPC.SendMissile(tgx, tgy : Byte; mtype : Byte);
const HITDELAY  = 50;
var Dist : byte;
    Roll : integer;
    Ray  : TRay;
    OldX : Word;
    OldY : Word;
    Scan      : Word;
    DrawDelay : Word;
begin
  Dist := Distance(x,y,tgx,tgy);
  DrawDelay := 10;
  case mtype of
      mtArrow : DrawDelay := 20;
  end;

{
  if Dist > 4 then
  begin
    Roll := RollDice;
    if Roll > Dexterity-((Dist-5) div 3) then
      if Dist < 9 then
      begin
        tgx := tgx + Random(3)-1;
        tgy := tgy + Random(3)-1;
      end else
      begin
        tgx := tgx + Random(5)-2;
        tgy := tgy + Random(5)-2;
      end;
  end;
}

  Ray.Init(x,y,tgx,tgy);
  repeat
    OldX := Ray.GetX;
    OldY := Ray.GetY;
    Ray.Next;
    //hit obstacle
    if not TLevel(Parent).CellCheck(Ray.GetX, Ray.GetY, [noObstacles]) then // Missile hits non-passable feature
      if TLevel(Parent).isVisible(Ray.GetX, Ray.GetY) then
      begin
        Output.DrawChar(Ray.GetX-TLevel(Parent).ShiftX+1, Ray.GetY-TLevel(Parent).ShiftY+2, LightGray, '*');
        Output.Update;
        Sleep(HITDELAY);
        TLevel(Parent).Draw;
        Break;
      end
      else
        Break;
   //hit monster
   if not TLevel(Parent).CellCheck(Ray.GetX, Ray.GetY, [noMonsters]) then
    begin
        case mtype of
            mtArrow  : if isPlayer then
                         TPlayer(self).Attack(TLevel(Parent).Map[Ray.GetX,Ray.GetY].NPC, true)
                       else
                         Attack(TLevel(Parent).Map[Ray.GetX,Ray.GetY].NPC);
        end;
        Output.DrawChar(Ray.GetX-TLevel(Parent).ShiftX+1, Ray.GetY-TLevel(Parent).ShiftY+2, Red,'*');
        Output.Update;
        Sleep(HITDELAY);
        TLevel(Parent).Draw;
        Break;
    end;

//    if mtype = MTBOMB then
//      if Ray.Done then Break;

    if TLevel(Parent).isVisible(Ray.GetX,Ray.GetY) then // Draw when visible
    begin
      case MTYPE of
          mtArrow : Output.DrawChar(Ray.GetX-TLevel(Parent).ShiftX+1, Ray.GetY-TLevel(Parent).ShiftY+2, Brown, DirPicture(DirCreate(OldX, OldY, Ray.GetX, Ray.GetY)));
      end;
      Output.Update;
      Sleep(DRAWDELAY);
      TLevel(Parent).Draw;
    end;
  until False;
//if mtype = MTBOMB then Level.Explosion(Ray.GetX,Ray.GetY,Red,4,4);}
end;


procedure TNPC.Action;
var dx : Real;
begin
  if Visible then
  begin
    tx := Game.Player.X;
    ty := Game.Player.Y;
  end;
  if (Abs(Game.Player.x-x) < 2) and (Abs(Game.Player.y-y) < 2) then
    Attack(Game.Player)
  else
    if not ((tx = x) and (ty = y)) then Seek(tx,ty);
  if SpeedCount >= 100 then Dec(SpeedCount,40);
end;

end.
