// @abstract(Player for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//

unit rlplayer;
interface
uses vutil, rlglobal, rlnpc, rlitem, rlthing, vrltools;

type TInventory = array[1..MaxInvSize] of TItem;
type TEquipment = array[1..MaxEqSlots] of TItem;

const
  // Target mode for ChooseTarget - look mode
  TM_LOOK  = 1;
  // Target mode for ChooseTarget - fire mode
  TM_FIRE  = 2;

const StatStr = 1;
      StatMag = 2;
      StatDex = 3;
      StatVit = 4;
      StatLife = 5;
      StatMana = 6;
      {Only for items:}
      StatAC   = 7;
      StatDmg  = 8;

type

{ TPlayer }

TPlayer = class(TNPC)
       klass   : byte;
       Inv     : TInventory;
       Eq      : TEquipment;
       InvMode : (None,Inventory,Equipment,InvChar,EqChar);
       Stats   : array[1..6] of Word;
       Exp     : LongInt;
       Mana    : Word;
       LevelUp : Boolean;
       Points  : Byte;
       PortalLevel: Byte;
       PortalCount: Byte;
       Querry     : TNPC;

       QuickSlots : array[1..COMMAND_QUICKSLOTSCOUNT] of TItem;
       // Overriden constructor
       constructor Create(const thingID :string); override;
       // Death procedure -- does all cleanup.
       procedure Die; override;
       // Command Act
       procedure doAct;
       // Command Inventory
       procedure doInv;
       // QuickSlot view
       procedure doQuickSlot;
       // Command Pickup
       procedure doPickUp;
       // DrawStatus bar
       procedure drawStatus;
       // Player action
       procedure Action; override;
       // Player Attack monster
       procedure Attack(NPC : TNPC; Ranged: boolean = false);
       // Block chance
       function getBlock : Word;
       // Damage base
       function getDmgBase : Word;
       function getDmgBaseRanged : Word;
       // returns DmgMin {with Items and charbonus}
       function getDmgMin : Word; override;
       // returns DmgMax {with Items and charbonus}
       function getDmgMax : Word; override;
       // returns AC {with Items}
       function getAC : Word; override;
       // returns ToHit{with Items}
       function getToHit : Word; override;
       // returns ToHit {with clvl and class bonus}
       function getToHitMelee : Word;
       // returns ToHit {with clvl and class bonus}
       function getToHitMagic : Word;
       // returns ToHit {with clvl and class bonus}
       function getToHitRange(Distance : Word) : Word;
       // returns HPMax
       function getLife : Word; override;
       function getMana : Word;
       function getStat(Stat : Byte) : Word;
       function getStr : Word;
       function getMag : Word;
       function getDex : Word;
       function getVit : Word;
       function getGold : DWord;
       function getItemStatBonus(Stat : Byte) : Integer;
       // Takes care of the items.
       destructor Destroy; override;
       // Adds given amount of gold. Returns number of pieces that
       // didn't fit.
       function addGold(Amount : DWord) : DWord;
       // Adds given amount of experience.
       procedure addExp(Amount : DWord);
       procedure UpdateStatus(Target : TThing);
       procedure durabilityCheck(Slot : Byte);
       procedure WriteMemorial;
       function SlotName(slot : byte) : string;
       function ItemToSlot(Item : TItem) : byte;
       procedure InvSort;
       function InvSize : Byte;
       function InvVolume : Word;
       function InvFreeSlot : Byte;
       function InvGoldSlot : Byte;
       procedure GenerateStartingGear;
       procedure EqDraw;
       procedure InvDraw;
       procedure ChaDraw;
       // checks wether item can be weared
       function ItemCheck(item : TItem) : Boolean;
       // Scrolls and Potions only -- uses the item and disposes of it.
       // Returns wether successful. If successful var argument set to nil.
       function UseItem(var item : TItem) : Boolean;
       // Interface function for choosing target. Returns true when in Fire mode
      // target is accepted.
      function ChooseTarget(TargetMode : Byte = TM_LOOK; FireCmd : byte = COMMAND_FIRE ) : boolean;
      function EnemiesAround : byte;

     end;

implementation
uses sysutils, vinput, voutput, vmath, rllevel, rlcell, rlmsg, rlgame, diablout;

const SlotNone    = 0;
      SlotFail2H  = 100;
      SlotFailReq = 101;
      SlotPotion  = 102;
      SlotScroll  = 103;
      SlotFailTown= 104;

function TPlayer.enemiesAround : byte;
var sx,sy : Byte;
begin
  enemiesAround := 1;
  for sx := x-1 to x+1 do
    for sy := y-1 to y+1 do
      if TLevel(Parent).Map[sx,sy].NPC <> nil then   Inc(enemiesAround);
  dec(enemiesAround);
end;

function TPlayer.ChooseTarget( TargetMode : Byte = TM_LOOK; FireCmd : byte = COMMAND_FIRE ) : boolean;
var Key,Dir : Byte;
begin

    if Eq[SlotRhand]<>nil then
    begin
      if (TargetMode=TM_FIRE) and (not(ifRanged in Eq[SlotRhand].flags)) then
      begin
        Msg.Add('What?');
        Exit(False);
      end;
    end
    else
    begin
      Msg.Add('What?');
      Exit(False);
    end;

    if (TargetMode<>TM_LOOK) and (LevelNumber = 0) then
    begin
      Msg.Add('Not in town!');
      Exit(False);
    end;

    tx := x;
    ty := y;

    if (TargetMode<>TM_LOOK) then
    begin
      if Querry = nil then
        Querry:=ClosestTarget([AIMonster],8)
      else if not (querry.Visible) then
        Querry:=ClosestTarget([AIMonster],8);
      if Querry <> nil then
      begin
        tx := Querry.x;
        ty := Querry.y;
      end;
    end;

    Output.MoveCursor(tx-TLevel(Parent).ShiftX+1, ty-TLevel(Parent).ShiftY+2);
    Output.ShowCursor;
  repeat
    TLevel(Parent).Draw;
//    if TargetMode = TM_FIRE then
//      UI.Target(tx,ty,RED);
    Output.MoveCursor(tx-TLevel(Parent).ShiftX+1, ty-TLevel(Parent).ShiftY+2);
    if not(TLevel(Parent).CellCheck(tx,ty,[noMonsters])) then
      UpdateStatus(TLevel(Parent).Map[tx,ty].NPC)
    else if not(TLevel(Parent).CellCheck(tx,ty,[noItems])) then
      UpdateStatus(TLevel(Parent).Map[tx,ty].Item)
    else
      UpdateStatus(nil);

    Output.Update;

    if TargetMode  in [TM_FIRE] then
    begin
      Output.DrawChar(tx-TLevel(Parent).ShiftX+1, ty-TLevel(Parent).ShiftY+2, Red, 'X');
      Key := Input.GetCommand(COMMANDS_MOVE+[COMMAND_ESCAPE,FireCmd]);
    end
    else
      Key := Input.GetCommand(COMMANDS_MOVE+[COMMAND_ESCAPE]);
    if Key = COMMAND_ESCAPE then
    begin
      tx := 0;
      Msg.Clear;
      Output.HideCursor;
      Exit(False);
    end;

    if (Key in COMMANDS_MOVE) then
    begin
      Dir := CommandDirection(Key);
      tx := tx + DirXValue(Dir);
      ty := ty + DirYValue(Dir);
    end;
  until Key in [FireCmd];

  if (x=tx)and(y=ty)then
  begin
    Msg.Add('Suicide? Too easy for You...');
    Output.HideCursor;
    exit(false);
  end;
  Dec(SpeedCount, SpdAtk);
  SendMissile(tX, tY, mtArrow);

  Output.HideCursor;
  Exit(True);
end;

procedure TPlayer.GenerateStartingGear;
begin
  Inv[1] := TItem.Create('gold');
  Inv[1].Amount := 100;
  case Klass of
    KlassWarrior  : begin
                      Eq[SlotRHand] := TItem.Create('shortsword');
                      Eq[SlotLHand] := TItem.Create('buckler');
                      Inv[2] := TItem.Create('club');
                      QuickSlots[1] := TItem.Create('pofhealing');
                      QuickSlots[2] := TItem.Create('pofhealing');
                    end;
    KlassRogue    : begin
                      Eq[SlotRHand] := TItem.Create('shortbow');
                      QuickSlots[1] := TItem.Create('pofhealing');
                      QuickSlots[2] := TItem.Create('pofhealing');
                    end;
    KlassSorceror : ;
  end;
end;


function TPlayer.SlotName(slot : byte) : string;
begin
  case Slot of
    SlotHead   : Exit('Head  ');
    SlotAmulet : Exit('Neck  ');
    SlotTorso  : Exit('Torso ');
    SlotRHand  : Exit('Weapon');
    SlotLHand  : Exit('Shield');
    SlotRRing  : Exit('RRing ');
    SlotLRing  : Exit('LRing ');
  end;
end;

function TPlayer.ItemToSlot(Item : TItem) : byte;
begin
  if Item = nil then Exit(SlotNone);
  if not ItemCheck(Item) then Exit(SlotFailReq);

  if ifOneHanded in Item.Flags then Exit(SlotRHand);
  if ifShield    in Item.Flags then
    if Eq[SlotRHand] = nil then Exit(SlotLHand)
                           else if ifTwoHanded in Eq[SlotRHand].Flags then Exit(SlotFail2H)
                                                                      else Exit(SlotLHand);
  if ifHelm      in Item.Flags then Exit(SlotHead);
  if ifArmor     in Item.Flags then Exit(SlotTorso);
  if ifAmulet    in Item.Flags then Exit(SlotAmulet);
  if ifTwoHanded in Item.Flags then
    if Eq[SlotLHand] = nil then Exit(SlotRHand)
                           else Exit(SlotFail2H);
  if ifRing      in Item.Flags then
    if Eq[SlotRRing] = nil then Exit(SlotRRing) else
      if Eq[SlotLRing] = nil then Exit(SlotLRing) else
         Exit(SlotRRing);

  if ifNotInTown in Item.Flags then
    if LevelNumber = 0 then Exit(SlotFailTown);
  if ifScroll    in Item.Flags then Exit(SlotScroll);
  if ifPotion    in Item.Flags then Exit(SlotPotion);
  Exit(SlotNone);
end;

function TPlayer.InvSize : Byte;
var Count : byte;
    Size  : byte;
begin
  Size := 0;
  for Count := 1 to MaxInvSize do
    if Inv[Count] <> nil then Inc(size);
  Exit(Size);
end;

function TPlayer.InvVolume : Word;
var Count  : byte;
    Volume : Word;
begin
  Volume := 0;
  for Count := 1 to MaxInvSize do
    if Inv[Count] <> nil then Inc(Volume,Inv[Count].Volume);
  Exit(Volume);
end;

function TPlayer.InvFreeSlot : Byte;
var Count : byte;
begin
  for Count := 1 to MaxInvSize do
    if Inv[Count] = nil then Exit(Count);
  Exit(0);
end;

function TPlayer.InvGoldSlot : Byte;
var Count : byte;
begin
  for Count := 1 to MaxInvSize do
    if Inv[Count] <> nil then
      if ifGold in Inv[Count].Flags then Exit(Count);
  Exit(0);
end;

function TPlayer.getGold : DWord;
var slot : Byte;
begin
  slot := InvGoldSlot;
  if slot = 0 then Exit(0) else Exit(Inv[slot].Amount);
end;


function TPlayer.getStat(Stat : Byte) : Word;
begin
  case Stat of
    StatVit : Exit(getVit);
    StatStr : Exit(getStr);
    StatMag : Exit(getMag);
    StatDex : Exit(getDex);
    StatLife: Exit(getLife);
    StatMana: Exit(getMana);
  end;
end;


function TPlayer.getVit : Word;
begin
  Exit(Stats[StatVit]+getItemStatBonus(statVit));
end;

function TPlayer.getStr : Word;
begin
  Exit(Stats[StatStr]+getItemStatBonus(statStr));
end;

function TPlayer.getMag : Word;
begin
  Exit(Stats[StatMag]+getItemStatBonus(statMag));
end;

function TPlayer.getDex : Word;
begin
  Exit(Stats[StatDex]+getItemStatBonus(statDex));
end;

function TPlayer.getAC : Word;
begin
  Exit(getDex div 5+getItemStatBonus(statAC));
end;

// Block chance
function TPlayer.getBlock : Word;
begin
  case Klass of
    KlassWarrior  : Exit(getDex+20);
    KlassRogue    : Exit(getDex+20);
    KlassSorceror : Exit(getDex+10);
  end;
end;


function TPlayer.getToHit : Word;
begin
  Exit(50 + getDex div 2{+items});
end;

function TPlayer.getToHitMelee : Word;
begin
  case Klass of
    KlassWarrior  : Exit(getToHit+Level+20);
    KlassRogue    : Exit(getToHit+Level);
    KlassSorceror : Exit(getToHit+Level);
  end;
end;

function TPlayer.getToHitMagic : Word;
begin
  case Klass of
    KlassWarrior  : Exit(50+getMag);
    KlassRogue    : Exit(50+getMag);
    KlassSorceror : Exit(50+getMag+20);
  end;
end;

function TPlayer.getToHitRange(Distance : Word) : Word;
begin
  case Klass of
    KlassWarrior  : Exit(Max(50+getDex{+Tohititems}+Level-((Distance*Distance) div 2)+10,0));
    KlassRogue    : Exit(Max(50+getDex{+Tohititems}+Level-((Distance*Distance) div 2)+20,0));
    KlassSorceror : Exit(Max(50+getDex{+Tohititems}+Level-((Distance*Distance) div 2)   ,0));
  end;
end;

// returns HPMax
function TPlayer.getLife : Word;
begin
  case Klass of
    KlassWarrior  : Stats[StatLife] := 2*Stats[StatVit]
                                     + 2*getItemStatBonus(statVit)
                                     + getItemStatBonus(statLife)
                                     + 2*Level + 18;
    KlassRogue    : Stats[StatLife] := 1*Stats[StatVit]
                                     + Round(1.5*getItemStatBonus(statVit))
                                     + getItemStatBonus(statLife)
                                     + 2*Level+23;
    KlassSorceror : Stats[StatLife] := 1*Stats[StatVit]
                                     + getItemStatBonus(statVit)
                                     + getItemStatBonus(statLife)
                                     + Level+9;
  end;
  HPMax := Stats[StatLife];
  Exit(HPMax);
end;

function TPlayer.getMana : Word;
begin
  case Klass of
    KlassWarrior  : Stats[StatMana] := 1*Stats[StatMag]
                                     + getItemStatBonus(statMag)
                                     + getItemStatBonus(statMana)
                                     + Level - 1;
    KlassRogue    : Stats[StatMana] := 1*Stats[StatMag]
                                     + Round(1.5*getItemStatBonus(statMag))
                                     + getItemStatBonus(statMana)
                                     + 2*Level + 5;
    KlassSorceror : Stats[StatMana] := 2*Stats[StatMag]
                                     + 2*getItemStatBonus(statMag)
                                     + getItemStatBonus(statMana)
                                     + 2*Level - 2;
  end;
  Exit(Stats[StatMana]);
end;

// returns Dmg
function TPlayer.getDmgBase : Word;
begin
  if Eq[SlotRhand]<>nil then
    if ifRanged in Eq[SlotRhand].flags then
      Exit(getDmgBaseRanged);
  case Klass of
     KlassWarrior,
     KlassSorceror  : Exit((getStr*Level) div 100);
     KlassRogue     : Exit(((getStr+getDex)*Level) div 200);
  end;
end;

function TPlayer.getDmgBaseRanged : Word;
begin
  case Klass of
     KlassWarrior,
     KlassSorceror  : Exit((getStr*Level) div 200);
     KlassRogue     : Exit(((getStr+getDex)*Level) div 200);
  end;
end;

// returns DmgMin
function TPlayer.getDmgMin : Word;
begin
  if Eq[SlotRHand] <> nil then
    Exit(getDmgBase+1+Eq[SlotRHand].DmgMin{+%}+getItemStatBonus(statDmg))
  else Exit(getDmgBase+1{+%}+getItemStatBonus(statDmg));
end;

// returns DmgMax
function TPlayer.getDmgMax : Word;
begin
  if Eq[SlotRHand] <> nil then
    Exit(getDmgBase+1+Eq[SlotRHand].DmgMax{+%}+getItemStatBonus(statDmg))
  else Exit(getDmgBase+1{+%}+getItemStatBonus(statDmg));
end;


function TPlayer.getItemStatBonus(Stat : Byte) : Integer;
var Count : byte;
    Bonus : Integer;
begin
  Bonus := 0;
  for Count := 1 to MaxEqSlots do
    if Eq[Count] <> nil then
      Bonus := Bonus + Eq[Count].StatChange[Stat];
  Exit(Bonus);
end;


procedure TPlayer.InvSort;
var cn   : byte;
    ci   : byte;
    icn  : byte;
    function Greater(Item1,Item2 : TItem) : Boolean;
    begin
      if Item1 = nil then Exit(True);
      if Item2 = nil then Exit(False);
      if Item1.Color > Item2.Color then Exit(True);
      if Item1.Color < Item2.Color then Exit(False);
      if Ord(Item1.Pic) > Ord(Item2.Pic) then Exit(False);
      if Ord(Item1.Pic) < Ord(Item2.Pic) then Exit(True);
      Exit(False);
    end;
    procedure Swap(var Item1,Item2 : TItem);
    var temp : TItem;
    begin
      temp := Item1;
      Item1 := Item2;
      Item2 := temp;
    end;
begin
  for cn := 1 to MaxInvSize-2 do
    for ci := 1 to MaxInvSize-cn do
      if Greater(Inv[ci],Inv[ci+1]) then Swap(Inv[ci],Inv[ci+1]);
end;

procedure TPlayer.EqDraw;
var Count : Word;
begin
  for Count := 1 to MaxEqSlots do
    if Eq[Count] = nil then Output.DrawString(4,Count+3,LightGray,'@d'+SlotName(Count)+' :@> ---')
                       else Output.DrawString(4,Count+3,LightGray,'@d'+SlotName(Count)+' :@> '+DPad(Eq[Count].GetName(PlainName),25));
end;

procedure TPlayer.InvDraw;
var Count : Word;
begin
  Count := 1;
  if invSize <> 0 then
  repeat
    if Inv[Count] <> nil then Output.DrawString(4,11+Count,DarkGray,Inv[Count].GetName(InvName));
    Inc(Count);
    if Count > 10 then Break;
  until False
  else Output.DrawString(3,12,DarkGray,' < @lEMPTY@> > ');
end;


procedure TPlayer.Die;
begin
  Msg.Add('You die... Press <@<Enter@>>...');
  Msg.Draw;
  Output.Update;
  Input.GetKey([VKEY_ENTER]);
  WriteMemorial;
  GameEnd := True;
end;


procedure TPlayer.ChaDraw;
var Window  : TWindow;
    Key     : Byte;
    function DualString(Stat,MaxStat : Integer) : string;
    begin
      if Stat = MaxStat then
      case length(IntToStr(Stat)) of
        1  : Exit('  @l'+IntToStr(Stat)+'@>|@l'+IntToStr(MaxStat)+'@>');
        2  : Exit(' @l'+IntToStr(Stat)+'@>|@l'+IntToStr(MaxStat)+'@>');
        else Exit('@l'+IntToStr(Stat)+'@>|@l'+IntToStr(MaxStat)+'@>');
      end;
      if Stat < MaxStat then
      case length(IntToStr(Stat)) of
        1  : Exit('  @l'+IntToStr(Stat)+'@>|@B'+IntToStr(MaxStat)+'@>');
        2  : Exit(' @l'+IntToStr(Stat)+'@>|@B'+IntToStr(MaxStat)+'@>');
        else Exit('@l'+IntToStr(Stat)+'@>|@B'+IntToStr(MaxStat)+'@>');
      end;
      if Stat > MaxStat then
      case length(IntToStr(Stat)) of
        1  : Exit('  @l'+IntToStr(Stat)+'@>|@R'+IntToStr(MaxStat)+'@>');
        2  : Exit(' @l'+IntToStr(Stat)+'@>|@R'+IntToStr(MaxStat)+'@>');
        else Exit('@l'+IntToStr(Stat)+'@>|@R'+IntToStr(MaxStat)+'@>');
      end;
    end;
    function StatString(Stat : byte) : string;
    begin
      Exit(DualString(Stats[Stat],getStat(Stat)));
    end;
begin
  if LevelUp then InvMode := None;
  if InvMode = None then
  begin
    Output.Clear;
    Output.DrawString(2,2,DarkGray,'                                                                              ');
    Msg.Clear;
    TLevel(Parent).DrawLeft;
  end;
  Window := TWindow.Init(41,3,79,22,DiabloStyle);
  Window.SetTitle('Character');
  if LevelUp then
  begin
    Output.DrawString(2,2,DarkGray,'LevelUp - press [@<s@>] Strength, [@<m@>] Magic, [@<d@>] Dexterity. [@<v@>] for Vitality.');
    Window.SetTitle('Level Up!');
  end;
  Window.Show;
  Window.Draw;
  repeat
  if Points = 0 then LevelUp := False;
  if not LevelUp then begin Window.SetTitle('Character'); Window.Draw;    Output.DrawString(2,2,DarkGray,'                                                                              '); end;
  Output.DrawString(43, 5,DarkGray,'Name                Class    ');
  Output.DrawString(43, 6,DarkGray,'Level               Exp      ');
  Output.DrawString(43, 7,DarkGray,'                    NextLev  ');

  Output.DrawString(43, 9,DarkGray,'Strength            Gold     ');
  Output.DrawString(43,10,DarkGray,'Magic               Armor    ');
  Output.DrawString(43,11,DarkGray,'Dexterity           ToHit    ');
  Output.DrawString(43,12,DarkGray,'Vitality            Damage   ');
  Output.DrawString(43,13,DarkGray,'Points             ');
  Output.DrawString(43,14,DarkGray,'                    ResMagic     @l5%@>');
  Output.DrawString(43,15,DarkGray,'Life                ResFire      @l5%@>');
  Output.DrawString(43,16,DarkGray,'Mana                ResLight     @l5%@>');
  Output.DrawString(43,17,DarkGray,'                    ');

  Output.LeftDrawString(58, 5,LightGray,Name);
  Output.LeftDrawString(58, 6,LightGray,IntToStr(Level));

  Output.LeftDrawString(77, 5,LightGray,UpCase(KlassName(Klass)));
  Output.LeftDrawString(77, 6,LightGray,IntToStr(Exp));
  Output.LeftDrawString(77, 7,LightGray,IntToStr(ExpTable[Level+1]));

  Output.LeftDrawString(77, 9,LightGray,IntToStr(getGold));
  Output.LeftDrawString(77,10,LightGray,IntToStr(getAC));
  Output.LeftDrawString(77,11,LightGray,IntToStr(getToHit));
{}{}{}{}  Output.LeftDrawString(77,12,LightGray,IntToStr(getDmgMin)+'-'+IntToStr(getDmgMax));

  Output.DrawString(53, 9,DarkGray,StatString(StatStr));
  Output.DrawString(53,10,DarkGray,StatString(StatMag));
  Output.DrawString(53,11,DarkGray,StatString(StatDex));
  Output.DrawString(53,12,DarkGray,StatString(StatVit));
{}{}{}{}  Output.DrawString(54,13,LightGray,IntToStr(Points));

  Output.DrawString(53,15,DarkGray,DualString(getLife,HP));
  Output.DrawString(53,16,DarkGray,DualString(getMana,Mana));
  if InvMode = None then DrawStatus;
  if InvMode = None then Output.Update;
  if LevelUp then
  begin
    Key := Input.GetKey([Ord('s'),Ord('m'),Ord('d'),Ord('v'), ord('c'), VKEY_ESCAPE]);
    case Key of
      Ord('s') : Inc(Stats[StatStr]);
      Ord('m') : Inc(Stats[StatMag]);
      Ord('d') : Inc(Stats[StatDex]);
      Ord('v') : Inc(Stats[StatVit]);
      VKEY_ESCAPE, Ord('c') : begin
                                InvMode := EqChar;
                                FreeAndNil(Window);
                                exit;
                              end;
    end;
    Dec(Points);
  end;
  until (Points = 0) and not LevelUp;
  LevelUp := False;
  FreeAndNil(Window);
end;

function TPlayer.ItemCheck(item : TItem) : Boolean;
var Count : Byte;
begin
  if Item = nil then Exit(False);
  for Count := 1 to 3 do
    if Item.StatReq[Count] > getStat(Count) then Exit(False);
  Exit(True);
end;

function TPlayer.UseItem(var item : TItem) : Boolean;
begin
  if (ifNotInTown in item.Flags) and (LevelNumber = 0) then
  begin
    Msg.Add('Not in town!');
    Exit(False);
  end;
       if ifPotion in item.Flags then Msg.Add('You drink '+item.GetName(TheName)+'.')
  else if ifScroll in item.Flags then Msg.Add('You read '+item.GetName(TheName)+'.')
  else Exit(False);
  DiabloPotion(item.Effect);
  Dec(SpeedCount,SpdMov);
  FreeAndNil(item);
  Exit(True);
end;


// Adds given amount of experience.
procedure TPlayer.addExp(Amount : DWord);
begin
  Exp := Exp + Amount;
  if Exp >= ExpTable[Level+1] then
  begin
    LevelUp := True;
    Inc(Level);
    Inc(Points,5);
    Msg.Add('You advance to @<level '+IntToStr(Level)+'@>!');
    HP   := getLife;
    Mana := getMana;
  end;
end;

// Player Attack monster
procedure TPlayer.Attack(NPC : TNPC; Ranged: boolean = false);
var HitChance  : Integer;
    Damage     : Integer;
begin
  if NPC = nil then Exit;
  if NPC = self then Exit;
  if NPC.AI = AINPC then
  begin
    if nfOnTalk in NPC.Flags then Game.Lua.Execute(NPC.tid+'_OnTalk');
    Exit;
  end;
  if not Ranged then
    Dec(SpeedCount,SpdAtk);
  Querry := NPC;

  if Ranged then
    HitChance := getToHitRange(Distance(x,y,NPC.x, NPC.y))
  else
    HitChance := getToHitMelee;

  HitChance := HitChance-NPC.getAC;

  if HitChance < 5  then HitChance := 5;
  if HitChance > 95 then HitChance := 95;

  if Random(100) >= HitChance then
  begin
    Msg.Add('You miss '+NPC.GetName(TheName)+'.');
    Exit;
  end;

  if Eq[SlotRHand] <> nil then
    Damage := getDmgBase+{+%}+getItemStatBonus(statDmg)+RandRange(Eq[SlotRHand].DmgMin,Eq[SlotRHand].DmgMax)
  else
    Damage := getDmgBase+1{+%}+getItemStatBonus(statDmg);

  // Critical hit
  if (Klass = KlassWarrior) and (Random(100) < Level)
    then begin Damage := 2*Damage;
               Msg.Add('You criticaly hit '+NPC.GetName(TheName)+'.');
               end
    else Msg.Add('You hit '+NPC.GetName(TheName)+'.');

  // Warning -- NPC may be invalid afterwards.
  NPC.ApplyDamage(Damage);
  durabilityCheck(SlotRHand);
end;


procedure TPlayer.durabilityCheck(Slot : Byte);
begin
  if Eq[slot] = nil then Exit;
  case slot of
    slotRHand : if Random(30)=0 then Dec(Eq[slot].Dur);
    slotTorso : if Random(4)<>0 then Dec(Eq[slot].Dur);
    slotHead  : if Random(4)<>0 then Dec(Eq[slot].Dur);
    slotLHand : if Random(10)=0 then Dec(Eq[slot].Dur);
  end;
  if Eq[slot].Dur = 0 then
  begin
    Msg.Add('Your '+Eq[slot].GetName(PlainName)+' breaks!');
    FreeAndNil(Eq[slot]);
  end;
end;

constructor TPlayer.Create(const thingID :string);
var Count : byte;
begin
  // Set other atributes and find given thingID
  inherited Create(thingID);
  klass := Database.GetNumber('klassid');
  for Count := 1 to MaxInvSize do Inv[Count] := nil;
  for Count := 1 to MaxEqSlots do Eq [Count] := nil;
  InvMode := None;
  Stats[StatStr] := Database.GetNumber('str');
  Stats[StatMag] := Database.GetNumber('mag');
  Stats[StatDex] := Database.GetNumber('dex');
  Stats[StatVit] := Database.GetNumber('vit');

  HP   := getLife;
  Mana := getMana;

  Name := 'Anubis';
  LevelUp := False;
  Querry := nil;
  PortalLevel:= 0;
  PortalCount:= 0;
  for Count := 1 to COMMAND_QUICKSLOTSCOUNT do QuickSlots[Count] := nil;

  GenerateStartingGear
end;

procedure TPlayer.doAct;
var Key : Byte;
begin
  Msg.Add('Choose direction...');
  Output.Update;
  Key := Input.GetCommand;
  case Key of
    COMMAND_WALKEAST   : ActivateCell(x+1,y);
    COMMAND_WALKWEST   : ActivateCell(x-1,y);
    COMMAND_WALKNORTH  : ActivateCell(x,y-1);
    COMMAND_WALKSOUTH  : ActivateCell(x,y+1);
    COMMAND_WALKNE     : ActivateCell(x+1,y-1);
    COMMAND_WALKNW     : ActivateCell(x-1,y-1);
    COMMAND_WALKSE     : ActivateCell(x+1,y+1);
    COMMAND_WALKSW     : ActivateCell(x-1,y+1);
  end;
  Dec(SpeedCount,SpdMov);
end;

procedure InvCallback(Choice : Word);
begin
  with Game.Player do
  begin
    Output.DrawString(2,2,LightGray,'                                                                              ');
    Output.DrawString(2,2,Inv[Choice].Color,'@lItem:@> '+Inv[Choice].GetName(AName));
    UpdateStatus(Inv[Choice]);
  end;
end;

procedure EqCallback(Choice : Word);
begin
with Game.Player do
begin
  Output.DrawString(2,2,LightGray,'                                                                              ');
  if Eq[Choice] <> nil then
  begin
    Output.DrawString(2,2,Eq[Choice].Color,'@lItem:@> '+Eq[Choice].GetName(AName));
  end;
  UpdateStatus(Eq[Choice]);
end;
end;

// Command Inventory
procedure TPlayer.doInv;
var Chooser : TWgtChoice;
    Window  : TWindow;
    Choice  : Byte;
    Slot    : Byte;
    cmd     : byte;
    Item    : TItem;
begin
  Output.Clear;
  Output.DrawString(2,2,DarkGray,'                                                                              ');
  if InvMode <> None then Output.DrawString(2,2,LightGray,Msg.Mem1);
  if InvMode = None then InvMode := Inventory;
  TLevel(Parent).DrawRight;
  Window := TWindow.Init(2,3,40,22,DiabloStyle);
  Window.SetTitle('Inventory');
  Window.Show;
  Window.Draw;
  Output.LeftDrawString(38,22,DarkGray,' @<@1@>/@2 ',[InvVolume,MaxVolume]);
  for choice := 3 to 39 do Output.DrawString(choice,11,DarkGray,'-');
  DrawStatus;

  Chooser := nil;
  repeat
    InvSort;
    if InvMode in [InvChar,EqChar] then
      if not LevelUp then ChaDraw
                     else begin ChaDraw; InvMode := None; Exit end;
    EqDraw;
    InvDraw;

    case InvMode of
    Inventory,InvChar : begin
          if Msg.Mem1 = '' then Output.DrawString(2,1,DarkGray,'Inventory mode: [@lTAB@>] switch, [@lENTER@>] equip/use, [@ld@>] drop, [@lESC@>] exit mode.');
          if InvSize = 0 then
          begin
            Output.DrawString(3,12,LightGray,' < EMPTY > ');
            Output.Update;
            choice :=  Input.GetKey([VKEY_ENTER,VKEY_ESCAPE,VKEY_TAB,Ord('c'),Ord('i')]);
            case choice of
              VKEY_TAB : if InvMode = InvChar then InvMode := EqChar
                                              else InvMode := Equipment;
              Ord('c') : if InvMode = InvChar then InvMode := Inventory
                                              else InvMode := InvChar;
            else InvMode := None;
            end;
            FreeAndNil(Window); Exit;
          end;
          Chooser := TWgtScrollChoice.Create(4,12,Black,DarkGray,White,10);
          Chooser.AddBreakChar(VKEY_ESCAPE);
          Chooser.AddBreakChar(VKEY_TAB);
          Chooser.AddBreakChar(Ord('d'));
          Chooser.AddBreakChar(Ord('i'));
          for Slot := 1 to COMMAND_QUICKSLOTSCOUNT do
          begin
            cmd := Input.CommandToVCode(COMMAND_QUICKSLOT1-1+Slot);
            if cmd > 0 then Chooser.AddBreakChar(cmd);
          end;
          Chooser.AddCallback(@InvCallback);
          Choice := 1;
          repeat
            Chooser.Add(Inv[choice].GetName(InvName));
            Inc(Choice);
            if Choice > MaxInvSize then Break;
          until Inv[Choice] = nil;
          cmd := Chooser.Run;
          Choice := Chooser.GetChoice;
          FreeAndNil(Chooser);
          Msg.Clear;
          if Input.VCodeToCommand(cmd) in COMMAND_QUICKSLOTS then cmd := Input.VCodeToCommand(cmd);
          case cmd of
            VKEY_TAB           : if InvMode = InvChar
                                   then InvMode := EqChar
                                   else InvMode := Equipment;
            VKEY_ESCAPE,
            Ord('i')           : begin
                                   InvMode := None;
                                   Break;
                                 end;
            Ord('d')           : begin
                                   Msg.Add('You drop '+Inv[Choice].GetName(TheName)+'.');
                                   Dec(SpeedCount,SpdMov);
                                   TLevel(Parent).DropItem(Inv[Choice],x,y);
                                   Inv[Choice] := nil;
                                   Break;
                                 end;
            Ord('c')           : if InvMode = InvChar
                                    then InvMode := Inventory
                                    else InvMode := InvChar;
            COMMAND_QUICKSLOT1 ..
            COMMAND_QUICKSLOT8    : if [ifPotion,ifScroll] * Inv[Choice].Flags <> [] then
                                   if QuickSlots[cmd-COMMAND_QUICKSLOT1+1] = nil then
                                   begin
                                     Msg.Add('You put '+Inv[Choice].GetName(TheName)+' behind your belt.');
                                     QuickSlots[cmd-COMMAND_QUICKSLOT1+1] := Inv[Choice];
                                     Dec(SpeedCount,SpdMov);
                                     Inv[Choice] := nil;
                                     Break;
                                   end
                                   else
                                   begin
                                     Item := QuickSlots[cmd-COMMAND_QUICKSLOT1+1];
                                     Msg.Add('You swap '+Inv[Choice].GetName(TheName)+' with '+Item.GetName(TheName)+'.');
                                     QuickSlots[cmd-COMMAND_QUICKSLOT1+1] := Inv[Choice];
                                     Dec(SpeedCount,SpdMov);
                                     Inv[Choice] := Item;
                                     Break;
                                   end
                                 else
                                 begin
                                   Msg.Add('Not a quickslot item!');
                                   Break;
                                 end;
            else
            begin
              slot := ItemToSlot(Inv[Choice]);
              case slot of
                1..MaxEqSlots : if Eq[slot] = nil then
                                  begin
                                    Msg.Add('You wear/wield '+Inv[Choice].GetName(TheName)+'.');
                                    Dec(SpeedCount,SpdMov);
                                    Eq[slot] := Inv[Choice];
                                    Inv[Choice] := nil;
                                    Break;
                                  end
                                else
                                  if Eq[slot].Volume <= MaxVolume-InvVolume+Inv[Choice].Volume then
                                    begin
                                      Item := Inv[Choice];
                                      Inv[Choice] := nil;
                                      Msg.Add('You take off '+Eq[slot].GetName(TheName)+'.');
                                      Inv[InvFreeSlot] := Eq[slot];
                                      Msg.Add('You wear/wield '+Item.GetName(TheName)+'.');
                                      Eq[slot] := Item;
                                      Dec(SpeedCount,2*SpdMov);
                                      Break;
                                    end;
                SlotPotion,
                SlotScroll    : begin
                                  UseItem(Inv[Choice]);
                                  Choice := 0;
                                  Break;
                                end;
                SlotFailTown  : Msg.Add('Not in town!');
                SlotFailReq   : Msg.Add('You do not meet the requirements!');
                else Msg.Add('What?');
              end;
            end;
          end;
        end;

    Equipment,EqChar : begin
          Output.DrawString(2,1,DarkGray,'Equipment mode: [@lTAB@>] switch, [@lENTER@>] unequip, [@ld@>] drop, [@lESC@>] exit mode.  ');
          Chooser := TWgtChoice.Create(4,4,Black,DarkGray,White);
          Chooser.AddBreakChar(VKEY_ESCAPE);
          Chooser.AddBreakChar(VKEY_TAB);
          Chooser.AddBreakChar(Ord('d'));
          Chooser.AddBreakChar(Ord('i'));
          Chooser.AddCallback(@EqCallback);
          for Choice := 1 to MaxEqSlots do
            Chooser.Add(SlotName(Choice));
          cmd := Chooser.Run;
          Choice := Chooser.GetChoice;
          FreeAndNil(Chooser);
          case cmd of
            VKEY_ESCAPE,Ord('i') : begin InvMode := None; Break; end;
            VKEY_TAB    : if InvMode = EqChar then InvMode := InvChar else InvMode := Inventory;
            Ord('d')    : if Eq[Choice] <> nil then
                          begin
                            Msg.Add('You drop '+Eq[Choice].GetName(TheName)+'.');
                            TLevel(Parent).DropItem(Eq[Choice],x,y);
                            Dec(SpeedCount,SpdMov);
                            Eq[Choice] := nil;
                            Break;
                          end;
             Ord('c')   : if InvMode = EqChar then InvMode := Equipment
                                              else InvMode := EqChar;
            else
             if Eq[Choice] <> nil then
             begin
               if Eq[Choice].Volume <= MaxVolume-InvVolume then
               begin
                 Msg.Add('You take off '+Eq[Choice].GetName(TheName)+'.');
                 Inv[InvFreeSlot] := Eq[Choice];
                 Dec(SpeedCount,SpdMov);
                 Eq[Choice] := nil;
                 Break;
               end;
             end;
          end;
        end;
    end; {CASE}
    FreeAndNil(Chooser);
  until False;
  FreeAndNil(Window);
end;

procedure TPlayer.doQuickSlot;
var Count : Byte;
    Window : TWindow;
    Key   : Byte;
begin
  Window := TWindow.Init(2,3,40,12,DiabloStyle);
  Window.SetTitle('Quickslots');
  Window.Show;
  Window.Draw;
  for Count := 1 to COMMAND_QUICKSLOTSCOUNT do
  begin
    Output.DrawString(4,3+Count,LightGray,Input.CommandToString(COMMAND_QUICKSLOT1-1+Count)+'@d -');
    if QuickSlots[Count] <> nil then
      Output.DrawString(9,3+Count,QuickSlots[Count].Color,QuickSlots[Count].GetName(AName))
    else
      Output.DrawString(9,3+Count,DarkGray,'none');
  end;
  Output.Update;
  Key := Input.VCodeToCommand(Input.GetKey);
  if Key in COMMAND_QUICKSLOTS then
    if QuickSlots[Key-COMMAND_QUICKSLOT1+1] <> nil then
      UseItem(QuickSlots[Key-COMMAND_QUICKSLOT1+1]);
  FreeAndNil(Window);
end;

function TPlayer.addGold(Amount : DWord) : DWord;
var slot : byte;
    rest : DWord;
begin
  slot := InvGoldSlot;
  if slot = 0 then
  begin
    if InvVolume >= MaxVolume then Exit(Amount);
    slot := InvFreeSlot;
    if slot = 0 then Exit(Amount);
    Inv[slot] := TItem.Create('gold');
    Inv[slot].Amount := Amount;
    exit(0);
  end;
  rest := GoldVolume-(Inv[slot].Amount mod GoldVolume);
  if rest = GoldVolume then rest := 0;
  if Amount <= rest then
  begin
    Inc(Inv[slot].Amount,Amount);
    Exit(0);
  end;
  Inc(Inv[slot].Amount,Rest);
  Dec(Amount,Rest);
  while Amount > 0 do
  begin
    if InvVolume >= MaxVolume then Exit(Amount);
    if Amount <= GoldVolume then
    begin
      Inc(Inv[slot].Amount,Amount);
      Inc(Inv[slot].Volume);
      Exit(0);
    end;
    Inc(Inv[slot].Amount,GoldVolume);
    Inc(Inv[slot].Volume);
    Dec(Amount,GoldVolume);
  end;
end;


procedure TPlayer.doPickUp;
var gold : DWord;
begin
  if TLevel(Parent).Map[x,y].Item = nil then
  begin
    Msg.Add('But there is nothing here!');
    exit;
  end;
  with TLevel(Parent).Map[x,y] do
    if ifGold in Item.Flags then
    begin
      if InvGoldSlot = 0 then
      begin
        if InvFreeSlot = 0 then
        begin
          Msg.Add('You ran out of place in your backpack.');
          exit;
        end;
        Msg.Add('You found '+Item.GetName(TheName)+'.');
        Dec(SpeedCount,SpdMov);
        Item.Detach;
        Inv[InvFreeSlot] := Item;
        Item := nil;
        exit;
      end;
      gold := AddGold(Item.Amount);
      if Gold = 0 then
      begin
        Msg.Add('You found '+Item.GetName(TheName)+'.');
        Dec(SpeedCount,SpdMov);
        Item.Detach;
        FreeAndNil(Item);
        exit;
      end;
      Msg.Add('You ran out of place in your backpack.');
      Dec(SpeedCount,SpdMov);
      Item.Amount := gold;
      exit;
    end;

  with TLevel(Parent).Map[x,y] do
  begin
    if (InvSize = MaxInvSize) or (InvVolume+Item.Volume > MaxVolume) then
    begin
      Msg.Add('You have no room in your backpack.');
      exit;
    end;
    Item.Detach;
    Inv[InvFreeSlot] := Item;
    Msg.Add('You put '+Item.GetName(TheName)+' into your backpack.');
    Dec(SpeedCount,SpdMov);
    TLevel(Parent).Map[x,y].Item := nil;
  end;
end;

procedure TPlayer.UpdateStatus(Target : TThing);
begin
  Output.DrawString(34,23,DarkGray, PadSpace('',12,'-'));
  Output.ClearRect(10,24,71,25);
  if Target = nil then Exit;
  if Target.inheritsFrom(TNPC) then
  begin
    TNPC(Target).DrawHP;
  end;
  if Target.inheritsFrom(TItem) then
  begin
    Output.CenterDrawString(40,24,TItem(Target).Color,TItem(Target).GetName(Status1));
    Output.CenterDrawString(40,25,TItem(Target).Color,TItem(Target).GetName(Status2));
  end;
end;

procedure TPlayer.Action;
var Key : Byte;
   procedure TryMove(nx,ny : shortint);
   begin with TLevel(Parent) do begin
     if Map[x+nx,y+ny].NPC <> nil then begin Dec(SpeedCount,SpdAtk); Attack(Map[x+nx,y+ny].NPC); Exit; end;
     if Cells[Map[x+nx,y+ny].Cell].Pass then
     begin Displace(x+nx,y+ny); Dec(SpeedCount,SpdMov); Querry:=nil; Exit; end;
     if Cells[Map[x+nx,y+ny].Cell].ActTo <> 0 then
     begin
       ActivateCell(x+nx,y+ny);
       Dec(SpeedCount,SpdMov);
     end;
   end; end;
begin
  Inc(PTurnCount);
  Output.Clear;
  // Again we assume that the parent is a TLevel
  Msg.Draw;
  DrawStatus;
  with TLevel(Parent) do
  begin
    LightClear;
    LightTrace(x,y,8);
    CalcShift(x,y);
    Draw;
    with Cells[Map[x,y].Cell] do
      if Special > 0 then
        DiabloCellSpecial(Special,x,y);

    // Portal possibility
    if LevelChange then
    begin
      Dec(SpeedCount,50);
      Exit;
    end;

    if Map[x,y].Item <> nil then
      with Map[x,y].Item do
        Msg.Add(Map[x,y].Item.GetName(CAName)+' is lying here.');
    if Map[x,y].Item <> nil then
      UpdateStatus(Map[x,y].Item)
    else if Querry<>nil then
      UpdateStatus(Querry)
    else
      UpdateStatus(nil);

  end;
  DrawStatus;
  if InvMode <> None then Key := COMMAND_INVENTORY
  else
  begin
    Output.Update;
    Key := Input.GetCommand;
  end;
  Msg.Update;
  case Key of
    COMMAND_QUIT       : begin GameEnd := True; Dec(SpeedCount,50); end;
    COMMAND_WALKEAST   : TryMove(+1,0);
    COMMAND_WALKWEST   : TryMove(-1,0);
    COMMAND_WALKNORTH  : TryMove(0,-1);
    COMMAND_WALKSOUTH  : TryMove(0,+1);
    COMMAND_WALKNE     : TryMove(+1,-1);
    COMMAND_WALKNW     : TryMove(-1,-1);
    COMMAND_WALKSE     : TryMove(+1,+1);
    COMMAND_WALKSW     : TryMove(-1,+1);
    COMMAND_ACT        : doAct;
    COMMAND_INVENTORY  : doInv;
    COMMAND_LOOK       : ChooseTarget;
    COMMAND_FIRE       : ChooseTarget(TM_FIRE, Key);
    COMMAND_WAIT       : Dec(SpeedCount,SpdMov);
    COMMAND_GOD1       : begin Hp := HpMax; Dec(SpeedCount,10); end;
    COMMAND_GOD2       : begin LevelChange := True; if LevelNumber < 16 then StairNumber := CellStairUp; Inc(LevelNumber); Dec(SpeedCount,50); end;
    COMMAND_GOD3       : addGold(5000);
    COMMAND_GOD4       : ;
    COMMAND_PLAYERINFO : begin ChaDraw; if LevelUp then exit; if Input.GetKey([Ord('i'),Ord('c'),VKEY_ESCAPE,VKEY_ENTER]) = Ord('i') then InvMode := InvChar; end;
    COMMAND_PICKUP     : doPickUp;
    COMMAND_STAIRSDOWN : if TLevel(Parent).Map[x,y].cell = CellStairDown then begin LevelChange := True; if LevelNumber < 16 then StairNumber := CellStairUp;   Inc(LevelNumber); Dec(SpeedCount,50); end;
    COMMAND_STAIRSUP   : if TLevel(Parent).Map[x,y].cell = CellStairUp   then begin LevelChange := True; if LevelNumber > 0  then StairNumber := CellStairDown; Dec(LevelNumber); Dec(SpeedCount,50); end;
    COMMAND_QUICKSLOT  : doQuickSlot;
    COMMAND_QUICKSLOT1 ..
    COMMAND_QUICKSLOT8 : if QuickSlots[Key-COMMAND_QUICKSLOT1+1] <> nil then
                           UseItem(QuickSlots[Key-COMMAND_QUICKSLOT1+1])
                         else
                         with TLevel(Parent).Map[x,y] do
                           if (Item <> nil) and ([ifPotion,ifScroll] * Item.Flags <> []) then
                           begin
                             Msg.Add('You found '+Item.GetName(AName)+'.');
                             Dec(SpeedCount,SpdMov);
                             Item.Detach;
                             QuickSlots[Key-COMMAND_QUICKSLOT1+1] := Item;
                             Item := nil;
                           end
                           else
                             Msg.Add('No quick item in slot '+IntToStr(Key-COMMAND_QUICKSLOT1+1)+'!');
    else Msg.Add('Unknown command. Read the help file!');
  end
end;

// DrawStatus bar
procedure TPlayer.DrawStatus;
var cnt : byte;
  function DurColor(Dur,MaxDur : Word) : Byte;
  begin
    if Dur >= (MaxDur div 4) then DurColor := LightGray
    else if Dur <= Max(MaxDur div 6,1) then DurColor := LightRed else
      DurColor := Brown;
  end;
  procedure DrawOrb(Value, ValueMax : Word; const color,x : Byte);
  var percent : Integer;
  begin
    percent := (Value * 100) div ValueMax;
    case percent of
      0..75   : Output.DrawString(x+1,22,color,'....');
      76..84  : Output.DrawString(x+1,22,color,'----');
      85..93  : Output.DrawString(x+1,22,color,'====');
      94..255 : Output.DrawString(x+1,22,color,'####');
    end;
    case percent of
      0..50   : Output.DrawString(x,23,color,'......');
      51..58  : Output.DrawString(x,23,color,'------');
      59..66  : Output.DrawString(x,23,color,'======');
      67..255 : Output.DrawString(x,23,color,'######');
    end;
    case percent of
      0..25   : Output.DrawString(x,24,color,'......');
      26..33  : Output.DrawString(x,24,color,'------');
      34..42  : Output.DrawString(x,24,color,'======');
      43..255 : Output.DrawString(x,24,color,'######');
    end;
    case percent of
      0..25   : Output.DrawString(x,24,color,'......');
      26..33  : Output.DrawString(x,24,color,'------');
      34..42  : Output.DrawString(x,24,color,'======');
      43..255 : Output.DrawString(x,24,color,'######');
    end;
    case percent of
      0..6    : Output.DrawString(x+1,25,color,'....');
      7..12   : Output.DrawString(x+1,25,color,'----');
      13..19  : Output.DrawString(x+1,25,color,'====');
      20..255 : Output.DrawString(x+1,25,color,'####');
    end;
  end;
begin
Output.DrawString(1,23,DarkGray,
'--------------------------------------------------------------------------------');
  for cnt := 1 to 80 do
  DrawOrb(HP,Stats[StatLife],Red,4);
  DrawOrb(10,10,Blue,72);
  Output.LeftDrawString(70,23,LightGray,' '+TLevel(Parent).Name+' ');
  if LevelUp then
  begin
    Output.DrawString(6,20,Red,'[@y+@r]');
    Output.DrawString(3,21,White,'Level up!');
  end;
  if Eq[slotHead] <> nil then
    with Eq[slotHead] do
    if DurColor(Dur,DurMax) <> LightGray then Output.DrawString(11,22,DurColor(Dur,DurMax),' helm ');
  if Eq[slotTorso] <> nil then
    with Eq[slotTorso] do
    if DurColor(Dur,DurMax) <> LightGray then Output.DrawString(18,22,DurColor(Dur,DurMax),' armor ');
  if Eq[slotRHand] <> nil then
    with Eq[slotRHand] do
    if DurColor(Dur,DurMax) <> LightGray then Output.DrawString(26,22,DurColor(Dur,DurMax),' weapon ');
  if Eq[slotLHand] <> nil then
    with Eq[slotLHand] do
    if DurColor(Dur,DurMax) <> LightGray then Output.DrawString(35,22,DurColor(Dur,DurMax),' shield ');
  Output.DrawString(11,23,DarkGray,' [');
  Output.DrawString(12+COMMAND_QUICKSLOTSCOUNT+1,23,DarkGray,'] ');
  for cnt := 1 to COMMAND_QUICKSLOTSCOUNT do
    if QuickSlots[cnt] = nil then Output.DrawChar(12+cnt,23,LightGray,'.')
                             else Output.DrawChar(12+cnt,23,QuickSlots[cnt].Color,QuickSlots[cnt].Pic);
end;


procedure TPlayer.WriteMemorial;
var VF : Text;
    VS : String;
    Count : Word;
    Score : LongInt;
begin
  Score := (Level * Level * 1000) + Exp + (TLevel(Parent).Level * 500) + getLife * 20;
  Assign(VF,'mortem.txt');
  Rewrite(VF);
  Writeln(VF,'--------------------------------------------------------------');
  Writeln(VF,' DiabloRL 7DRL PRERELEASE roguelike postmortem character dump');
  Writeln(VF,'--------------------------------------------------------------');
  Writeln(VF,'');
  if ScriptData['BUTCHERKILLED'] = 1 then
    Writeln(VF,'  '+Name+', level '+IntToStr(Level)+' '+LowerCase(KlassName(Klass))+', killed the Butcher')
  else
    Writeln(VF,'  '+Name+', level '+IntToStr(Level)+' '+LowerCase(KlassName(Klass))+', killed');
  Writeln(VF,'  in '+TLevel(Parent).Name+', after '+IntToStr(TurnCount div 4)+' turns.');
  Writeln(VF,'  '+KlassPronoun(Klass)+' scored '+IntToStr(Score)+' points, killing '+IntToStr(KillCount)+' hellspawn.');
  Writeln(VF,'');
  Writeln(VF,'  '+KlassPronoun(Klass)+' advanced to level '+IntToStr(Level)+' gainging '+IntToStr(Exp)+' experience.');
  Writeln(VF,'  '+KlassPronoun(Klass)+' found '+IntToStr(getGold)+' gold coins.');

  Writeln(VF,'');
  Writeln(VF,'-- Statistics ------------------------------------------------');
  Writeln(VF,'');
  Writeln(VF,'  Strength   '+IntToStr(Stats[StatStr])+'/'+IntToStr(getStr));
  Writeln(VF,'  Magic      '+IntToStr(Stats[StatMag])+'/'+IntToStr(getMag));
  Writeln(VF,'  Dexterity  '+IntToStr(Stats[StatDex])+'/'+IntToStr(getDex));
  Writeln(VF,'  Vitality   '+IntToStr(Stats[StatVit])+'/'+IntToStr(getVit));
  Writeln(VF,'');
  Writeln(VF,'  Life '+IntToStr(getLife)+'  Mana '+IntToStr(getMana));
  Writeln(VF,'  Armor '+IntToStr(getAC)+'  ToHit '+IntToStr(getToHit));
  Writeln(VF,'');
  Writeln(VF,'-- Equipment -------------------------------------------------');
  Writeln(VF,'');
  for Count := 1 to MaxEqSlots do
    if Eq[Count] = nil then Writeln(VF,'  [ '+SlotName(Count)+' ]  nothing')
                       else Writeln(VF,'  [ '+SlotName(Count)+' ] '+Eq[Count].GetName(PlainName));
  Writeln(VF,'');
  Writeln(VF,'-- Quickslots ------------------------------------------------');
  Writeln(VF,'');
  for Count := 1 to COMMAND_QUICKSLOTSCOUNT do
    if QuickSlots[Count] <> nil then Writeln(VF,'  [ Slot '+IntToStr(Count)+' ] '+QuickSlots[Count].GetName(PlainName));
  Writeln(VF,'');
  Writeln(VF,'-- Inventory -------------------------------------------------');
  Writeln(VF,'');
  Count := 1;
  if invSize <> 0 then
  repeat
    if Inv[Count] <> nil then Writeln(VF,'  '+Inv[Count].GetName(PlainName));
    Inc(Count);
  until Count >= MaxInvSize
  else Writeln(VF,' < EMPTY > ');
  Writeln(VF,'');
  Writeln(VF,'--------------------------------------------------------------');
  Close(VF);
end;

// Takes care of the items.
destructor TPlayer.Destroy;
var Count : byte;
begin
  for Count := 1 to COMMAND_QUICKSLOTSCOUNT do FreeAndNil(QuickSlots[Count]);
  for Count := 1 to MaxInvSize do FreeAndNil(Inv[Count]);
  for Count := 1 to MaxEqSlots do FreeAndNil(Eq [Count]);
  inherited Destroy;
end;

end.
