// @abstract(Thing Object for RL Core)
// @author(Kornel Kisielewicz <kisiel@fulbrightweb.org>)
// @created(January 17, 2005)
// @lastmod(January 17, 2005)
//
// This unit holds the TThing class. A TThing holds common data and
// methods for TItem and TNPC.

unit rlthing;
interface
uses rlgobj;

// Generic Thing class -- ancestor to both TItem and TNPC.
type TThing = class(TGameObject)
       x,y   : Word;
       Pic   : Char;
       Color : Byte;
       Level : Word;
       // Overriden constructor -- sets x,y to (1,1).
       constructor Create(const thingID :string); override;
       // We would love to name this one just Move, but Move is a
       // method of TNode. All manipulation on x,y should be done by
       // displace, cause it also displaces the Thing on the map!
       procedure Displace(newx,newy : Word);
       // TThing time flow. Currently just passes time flow to TGameObject.
       procedure TimeFlow(time : LongInt); override;
       // Overriden destructor, does nothing (yet)
       destructor Destroy; override;
     end;

implementation
uses rlnpc, rlitem, rllevel, rlglobal, voutput;

constructor TThing.Create(const thingID :string);
begin
  Pic := '?';
  Color := LightGray;
  // With TNode descendants ALWAYS call inherited Init and Done.
  inherited Create(thingID);
  // To be on the safe side we set x,y to 1, 'cause (0,0) is not
  // a valid map coordinate in this game.
  x := 1; y := 1;
  // A Displace must be called anyway, to set the Thing on the map!
  if Database.Defined('pic') then Pic := Database.GetString('pic')[1];
  if Database.Defined('color') then Color := Database.GetNumber('color');
  Level := Database.GetNumber('level');
end;

// TThing time flow. Currently just passes time flow to TGameObject.
procedure TThing.TimeFlow(time : LongInt);
begin
  inherited TimeFlow(time);
end;


procedure TThing.Displace(newx,newy : Word);
begin
  // Note that TThing's parent MUST be TLevel, or else we
  // can't access Parent in the way we do!
  with TLevel(Parent) do
  begin
    // We have to determinate wether we are a NPC or Item to
    // access the proper mapfields.
    if Self.inheritsFrom(TNPC) then
    begin
      if Map[x,y].NPC = Self then Map[x,y].NPC := nil;
      Map[newx,newy].NPC := TNPC(Self);
    end else
    if Self.inheritsFrom(TItem) then
    begin
      if Map[x,y].Item = Self then Map[x,y].Item := nil;
      Map[newx,newy].Item := TItem(Self);
    end
    else CritError('We have a TThing that is neither a TNPC nor a TItem!');
  end;
  // Now we can update the coordinates.
  x := newx; y := newy;
end;


destructor TThing.Destroy;
begin
  // With TNode descendants ALWAYS call inherited Init and Done.
  inherited Destroy;
end;

end.
