unit vedit;
interface
uses vutil;

const MaxEditLines = 10;
      EditAlphaNumerics = '1234567890qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM';
      EditSigns         = '!#$%^&*()_+-=[]{}\|;:''"<>,./?`~ ';
      EditChars         = EditAlphaNumerics + EditSigns;


type VEditLinePtr = ^VEditLine;
     VEditLine = object
                   Next : VEditLinePtr;
                   Str  : PString;
                   constructor Init(nStr : string; nNext : VEditLinePtr);
                   function    Return : String;
                   destructor  Done;
                 end;


type VEditFieldPtr = ^VEditField;
     VEditField = object
                    Data : array[1..MaxEditLines] of string[80];
                    PosX : byte;
                    PosY : byte;
                    Attr : byte;
                    Lines   : byte;
                    MLenght : byte;
                    constructor Init(x,y,nlenght, nlines, nattr : byte);
                    procedure   ShiftLine(ln,pos : byte; shift : ShortInt);
                    procedure   ShiftText(ln : byte; shift : ShortInt);
                    function    LineEmpty(s : string) : boolean;
                    function    Return(ln : byte) : string;
                    procedure   Run;
                    procedure   Refresh;
                    destructor  Done;
                  end;

implementation

uses SysUtils, vdebug, voutput,vinput;

constructor VEditLine.Init(nStr : string; nNext : VEditLinePtr);
begin
  Str  := NewStr(nStr);
  Next := nNext;
end;

function    VEditLine.Return : String;
begin
  Exit(Str^);
end;

destructor  VEditLine.Done;
begin
  DisposeStr(Str);
end;


constructor VEditField.Init(x,y,nlenght, nlines, nattr : byte);
var c : byte;
begin
  for c := 1 to MaxEditLines do Data[c] := '';
  PosX := x; PosY := y;
  Attr := nAttr;
  if Lines > MaxEditLines then CritError('EditField Lines parameter too big!');
  Lines := nLines;
  MLenght := nLenght;
  if MLenght + PosX > 80 then CritError('EditField out of bounds (X)!');
  if Lines   + PosY > 25 then CritError('EditField out of bounds (Y)!');
end;

function VEditField.Return(ln : byte) : string;
begin
  Exit(Data[ln]);
end;


function VEditField.LineEmpty(s : string) : boolean;
var k : byte;
begin
  if Length(s) = 0 then Exit(True);
  for k := 1 to Length(s) do if s[k] <> ' ' then Exit(False);
  Exit(True);
end;



procedure VEditField.Refresh;
var l : byte;
    b : boolean;
begin
  for l := 1 to Lines do
    Output.PrintString(PosX,PosY+l-1,Attr,Data[l]);
  Output.Update;
end;

procedure VEditField.ShiftText(ln : byte; shift : ShortInt);
var cn : byte;
begin
if Shift < 0 then
begin
  for cn := ln to Lines do
      Data[cn] := Data[cn+1];
  Data[Lines] := '';
  for cn := Length(Data[Lines]) to MLenght-1 do
    Data[Lines] := Data[Lines]+' ';
end
else
begin
end;
end;


procedure VEditField.ShiftLine(ln,pos : byte; shift : ShortInt);
var cn : byte;
begin
  if shift > 0 then
    for cn := MLenght downto pos+1 do
      Data[ln][cn] := Data[ln][cn-1];
  if shift < 0 then
  begin
    for cn := pos to MLenght-1 do
      Data[ln][cn] := Data[ln][cn+1];
    Data[ln][MLenght] := ' ';
  end;
end;

procedure VEditField.Run;
var x,y : byte;
    Key : byte;
begin
  Output.ShowCursor;
  for y := 1 to Lines do
    if Length(Data[y]) < MLenght then
    for x := Length(Data[y]) to MLenght-1 do
        Data[y] := Data[y]+' ';
  x := 1;
  y := 1;
  Refresh;
  Output.MoveCursor(PosX+x-1,PosY+y-1);
  repeat
    Key := Input.GetKey;
//    VFilterkey(EditChars+
  //                    VKUpArr+VkDownArr+VkLeftArr+VkRightArr
    //                  +VkHome+VkEnd
      //                +VKEscape);
    case Key of
      VKEY_BackSpace : if x > 1 then
               begin
                 Dec(x);
                 ShiftLine(y,x,-1);
               end else
                if y > 1 then
                  if LineEmpty(Data[y-1]) then begin Dec(y); ShiftText(y,-1); end;
      VKEY_Enter : if Lines = 1 then Key := VKEY_Escape;
      VKEY_Left  : if x > 1       then Dec(x);
      VKEY_Right : if x < MLenght then Inc(x);
      VKEY_Up    : if y > 1       then Dec(y);
      VKEY_Down  : if y < Lines   then Inc(y);
      VKEY_Home     : if x <> 1 then x := 1 else y := 1;
      VKEY_End      : if x <> MLenght then x := MLenght else y := Lines;
      VKEY_Delete   : if LineEmpty(Data[y]) and (x = 1) then ShiftText(y,-1) else ShiftLine(y,x,-1);
      else if Pos(chr(key),EditChars) <> 0 then
        begin
          ShiftLine(y,x,1);
          Data[y][x] := chr(key);
          if x < MLenght then Inc(x);
        end
    end;

    Output.MoveCursor(PosX+x-1,PosY+y-1);
    Refresh;
  until Key = VKEY_Escape;
end;


destructor VEditField.Done;
begin
end;

end.
