unit vluaio;
interface
uses lua;

procedure LoadLuaIO(L: Plua_State);
//procedure CleanLuaIO;

function WordOrDef(L: Plua_State; stackNum : byte; const def : Word) : Word;
function DWordOrDef(L: Plua_State; stackNum : byte; const def : DWord) : DWord;
function ByteOrDef(L: Plua_State; stackNum : byte; const def : Byte) : Byte;
function BoolOrDef(L: Plua_State; stackNum : byte; const def : Boolean) : Boolean;
function LuaStr(L: Plua_State; stackNum : byte) : string;

implementation
uses vutil,voutput,vinput;

const ColorNames : array[0..15] of PChar =
      ('BLACK',     'BLUE',     'GREEN',    'CYAN',        'RED',
       'MAGENTA',   'BROWN',    'LIGHTGRAY','DARKGRAY',    'LIGHTBLUE',
       'LIGHTGREEN','LIGHTCYAN','LIGHTRED', 'LIGHTMAGENTA','YELLOW',
       'WHITE');

procedure RegisterConst(L: Plua_State; name : PChar; value : Integer);
begin
  lua_pushnumber(L,value);
  lua_setglobal(L,name);
end;

function LuaStr(L: Plua_State; stackNum : byte) : string;
begin
  if lua_isstring(L,stackNum) or lua_isnumber(L,stackNum) then
  begin
    LuaStr := lua_tostring(L, stackNum);
    //Delete(LuaStr,Length(LuaStr),1);
  end else Exit('');
end;

function BoolOrDef(L: Plua_State; stackNum : byte; const def : Boolean) : Boolean;
begin
  if not lua_isboolean(L,stackNum) then Exit(Def);
  Exit(lua_toboolean(L, stackNum));
end;

function WordOrDef(L: Plua_State; stackNum : byte; const def : Word) : Word;
var Num : lua_Number;
begin
  if not lua_isnumber(L,stackNum) then Exit(Def);
  Num := lua_tonumber(L, stackNum);
  if Num >= 0 then Exit(Round(Num)) else Exit(Def);
end;

function DWordOrDef(L: Plua_State; stackNum : byte; const def : DWord) : DWord;
var Num : lua_Number;
begin
  if not lua_isnumber(L,stackNum) then Exit(Def);
  Num := lua_tonumber(L, stackNum);
  if Num >= 0 then Exit(Round(Num)) else Exit(Def);
end;

function ByteOrDef(L: Plua_State; stackNum : byte; const def : Byte) : Byte;
var Num : lua_Number;
begin
  if not lua_isnumber(L,stackNum) then Exit(Def);
  Num := lua_tonumber(L, stackNum);
  if (Num >= 0) and (Num <= 255) then Exit(Round(Num)) else Exit(Def);
end;


function lua_update(L: Plua_State): Integer; cdecl;
begin
  Output.Update;
  Result := 0;
end;

function lua_clear(L: Plua_State): Integer; cdecl;
begin
  Output.Clear;
  Result := 0;
end;

function lua_drawstring(L: Plua_State): Integer; cdecl;
var i, n: Integer;
    x,y : Word;
    col : byte;
begin
  n := lua_gettop(L);
  if n < 4 then Exit(0);
  x   := WordOrDef(L,1,1);
  y   := WordOrDef(L,2,1);
  col := WordOrDef(L,3,7);
  if n = 5 then
    if lua_isNumber(L,5) then
    begin
      Output.DrawString(x,y+i-4,col,LuaStr(L,4),WordOrDef(L,5,80));
      Exit(0);
    end;
  for i := 4 to n do
    Output.DrawString(x,y+i-4,col,LuaStr(L,i));
  Result := 0;
end;

function lua_leftdrawstring(L: Plua_State): Integer; cdecl;
var i, n: Integer;
    x,y : Word;
    col : byte;
begin
  n := lua_gettop(L);
  if n < 4 then Exit(0);
  x   := WordOrDef(L,1,80);
  y   := WordOrDef(L,2,1);
  col := ByteOrDef(L,3,7);
  for i := 4 to n do
    Output.LeftDrawString(x,y+i-4,col,LuaStr(L,i));
  Result := 0;
end;

function lua_centerdrawstring(L: Plua_State): Integer; cdecl;
var i, n: Integer;
    x,y : Word;
    col : byte;
begin
  n := lua_gettop(L);
  if n < 4 then Exit(0);
  x   := WordOrDef(L,1,40);
  y   := WordOrDef(L,2,1);
  col := ByteOrDef(L,3,7);
  for i := 4 to n do
    Output.CenterDrawString(x,y+i-4,col,LuaStr(L,i));
  Result := 0;
end;

function lua_getkey(L: Plua_State): Integer; cdecl;
var
  i, n : Integer;
  b    : Byte;
  kset : TKeyFilter;

begin
  n := lua_gettop(L);
  if n = 0 then
    b := Input.GetKey
  else
  begin
    kset := [];
    for i := 1 to n do
    begin
      if not lua_isnumber(L, i) then Continue;
      b := ByteOrDef(L,i,0);
      if b <> 0 then Include(kset,b);
    end;
    b := Input.GetKey(kset);
  end;
  
  lua_pushnumber(L,b);
  Result := 1;
end;


procedure LoadLuaIO(L: Plua_State);
var count : byte;
    pc    : AnsiString;
begin
  for Count := 0 to 15 do RegisterConst(L,ColorNames[Count],Count);
  for Count := 0 to 16 do
  begin
    pc := 'VKEY_'+Input.VCodeToString(Count)+#0;
    RegisterConst(L,PChar(pc),Count);
  end;
  lua_register(L, 'DrawString'      , @lua_drawstring);
  lua_register(L, 'LeftDrawString'  , @lua_leftdrawstring);
  lua_register(L, 'CenterDrawString', @lua_centerdrawstring);
  lua_register(L, 'Update'          , @lua_update);
  lua_register(L, 'Clear'           , @lua_clear);
  lua_register(L, 'GetKey'          , @lua_getkey);
end;

end.

