{*  Valkyrie v 0.0.1                             *}
{*  VViewer -- Coded Valkyrie Texts Viewer       *}
{*  Copyright (c) 2001 by Anubis Raskolnikov     *}
{*  Target : DOS (GO32V2)                        *}

unit vviewer;
interface
uses vutil, vgzstream, voutput;

const VVMaxLines = 2000;
      VVNormalColor : byte       = LightGray;
      VVBoldColor   : byte       = White;
      VVIBarColor   : byte       = Blue;
      VVITextColor  : byte       = LightBlue;
      VVRulerColor  : byte       = DarkGray;
      VVRulers      : boolean    = True;
      VVTitle       : string[80] = '';
      VVSub         : string[80] = '';
      VVHighASCII   : boolean    = True;
      VVEnterExits  : boolean    = False;


var OldVFPrepareStatus : Boolean;
type VFileDataPtr = ^VFileView;
     VFileView = object
                   FN      : string;
                   FBuf    : array[1..VVMaxLines] of AnsiString;
                   FLines  : Word;
                   LinePos : Word;
                   Coded   : Boolean;
                   procedure Init; // for external init
                   procedure Open(FName : string);
                   procedure Load(var vgz : TGZFileInputStream; Lines : Word);
                   procedure AddStr(S : shortstring);
                   procedure SinglePage;
                   procedure ScreenPrepare;
                   procedure UnCoded;
                   procedure Run;
                   procedure Done;
                 end;

implementation

uses SysUtils, vinput;

function Percent(Base,Max : LongInt) : string;
var p : Byte;
begin
 if (Max < 1) then Exit('100%');
  p := Trunc((Base / max  )*100);
  if p    = 0 then Exit('  1%');
  if p >= 100 then Exit('100%');
  if (p < 10) and (p > -1) then Exit('  '+IntToStr(p)+'%')
  else Exit(' '+IntToStr(p)+'%');
end;

procedure CodedWrite(s : string; x,y : Byte);
var cnt,pos : byte;
    col,LEN : word;
    skip    : boolean;
begin
  col := VVNormalColor;
  skip := false;
  pos  := 0;
  LEN := Length(s);
  for cnt := 1 to LEN do
  begin
   if Skip then begin Skip := False; Continue; end;
   if s[cnt] = '@' then
   begin
      case s[cnt+1] of
        '>' : col := VVNormalColor;
        '<' : col := VVBoldColor;
        'r' : col := Red;
        'R' : col := LightRed;
        'b' : col := Blue;
        'B' : col := LightBlue;
        'g' : col := Green;
        'G' : col := LightGreen;
        'v' : col := Magenta;
        'V' : col := LightMagenta;
        'c' : col := Cyan;
        'C' : col := LightCyan;
    'n','N' : col := Brown;
    'y','Y' : col := Yellow;
        'l' : col := LightGray;
        'L' : col := White;
        'd' : col := DarkGray;
        'D' : col := Black;
        '@' : begin
                Output.DrawChar(x+pos,y,col,s[cnt]);
                Inc(pos);
              end;
      end;
      Skip := True;
      Continue;
    end;
    if x+pos > 80 then Exit;
    Output.DrawChar(x+pos,y,col,s[cnt]);
    Inc(pos);
    If cnt >= LEN then Break;
  end;
end;

procedure VFileView.Init;
var w  : word;
begin
  for w := 1 to VVMaxLines do FBuf[w] := '';
  Coded := True;
  FLines := 0;
end;

procedure VFileView.Load(var vgz : TGZFileInputStream; Lines : Word);
var cnt : Word;
    VS  : String;
begin
  FLines := 0;
  for cnt := 1 to Lines do
  begin
    VS := vgz.ReadString;
    AddStr(VS);
  end
end;

procedure VFileView.AddStr(S : shortstring);
begin
  Inc(FLines);
  FBuf[FLines] := S;
end;

procedure VFileView.Open(FName : string);
var RS : shortstring;
    F  : Text;
begin
  FN := FName;
  Init;
  Assign(F,FName);
  Reset(F);
  FLines := 0;
  repeat
    Readln(F,RS);
    AddStr(RS);
  until EOF(F) or (RS = '$');
  Close(F);
  LinePos := 0;
end;

procedure VFileView.UnCoded;
begin
  Coded := False;
end;

procedure VFileView.ScreenPrepare;
var c  : byte;
    uc,
    dc,
    rc : char;
begin
  if VVHighASCII then begin uc := ''; dc := ''; rc := ''; end else begin uc := '^'; dc := 'v'; rc := '-' end;
  Output.Clear;
  if VVTitle = '' then CodedWrite('@BFile Viewer : @L'+FN,3,1)
                  else CodedWrite(VVTitle,3,1);
  if VVSub   = '' then CodedWrite('@BArrows, PgUp, PgDown, Home, End, Escape',3,25)
                  else CodedWrite(VVSub,3,25);
  Output.DrawString(70,1,VVITextColor,Percent(LinePos,FLines-21));

  for c := 1 to 21 do
    if FBuf[c+LinePos] <> '' then
      CodedWrite(FBuf[c+LinePos],3,2+c);

  if VVRulers then
  begin
    for c := 1 to 80 do Output.DrawChar(c,2,VVRulerColor,rc);
    for c := 1 to 80 do Output.DrawChar(c,24,VVRulerColor,rc);
  end;

  if LinePos <> 0 then Output.DrawChar(79,2,LightBlue,uc);
  if LinePos <> 0 then Output.DrawChar(2, 2,LightBlue,uc);
  if LinePos < (FLines-21) then Output.DrawChar(79,24,LightBlue,dc);
  if LinePos < (FLines-21) then Output.DrawChar(2 ,24,LightBlue,dc);
end;

procedure VFileView.SinglePage;
var c : byte;
    rc : char;
begin
  if VVHighASCII then rc := '' else rc := '-';
  Output.Clear;
  if VVTitle = '' then CodedWrite('@BFile Viewer : @L'+FN,3,1)
                  else CodedWrite(VVTitle,3,1);
  if VVSub   = '' then CodedWrite('@BPress <Enter> or <Escape>...',3,25)
                  else CodedWrite(VVSub,3,25);
  Output.DrawString(70,1,VVITextColor,'100%');
  for c := 1 to FLines do
    if FBuf[c+LinePos] <> '' then
      CodedWrite(FBuf[c+LinePos],3,2+c);
  if VVRulers then
  begin
    for c := 1 to 80 do Output.DrawChar(c,2,VVRulerColor,rc);
    for c := 1 to 80 do Output.DrawChar(c,24,VVRulerColor,rc);
  end;
  Output.Update;
  Input.GetKey([VKEY_Enter,VKEY_Escape]);
end;

procedure VFileView.Run;
var Key : byte;
begin
  Output.MoveCursor(1,1);
  if FLines < 22 then SinglePage else
  repeat
    ScreenPrepare;
    Output.Update;
    Key := Input.GetKey([VKEY_Up,VKEY_Down,VKEY_Escape,VKEY_Home,VKEY_End,VKEY_PageDown,VKEY_PageUp,VKEY_Enter]);
    case Key of
      VKEY_Up   : if LinePos > 0           then Dec(LinePos);
      VKEY_Down : if LinePos < (FLines-21) then Inc(LinePos);
      VKEY_Home : LinePos := 0;
      VKEY_End  : LinePos := FLines-21;
      VKEY_PageUp   : if LinePos > 20 then
                     Dec(LinePos,20) else LinePos := 0;
      VKEY_PageDown : if LinePos < (Flines-41) then
                     Inc(LinePos,20) else LinePos := FLines-21;
      VKEY_Enter : if VVEnterExits then Break;
    end;
  until Key = VKEY_Escape;
  Output.Clear;
  Output.Update;
end;

procedure VFileView.Done;
var w : word;
begin
  for w := 1 to VVMaxLines do FBuf[w] := '';
end;


end.

